<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: d6cd5ece8ef6 $
 * $Author: Lukas Imhof $
 * $Date: 2020-10-02 13:53 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_contents_objectBase extends we_document_base{
	protected const ELEMENT_LENGHT = 'length';
	public const ELEMENT_TYPE = 'dtype';
	protected const ELEMENT_HEIGHT = 'height';
	protected const ELEMENT_CLASS = 'class';
	protected const ELEMENT_MAX = 'max';
	protected const ELEMENT_DEFAULT = 'default';
	//public const TYPE_ALL = 'allFields';
	public const TYPE_GENERAL = 'general';
	public const TYPE_AUDIO = 'audio';
	public const TYPE_BINARY = 'binary';
	public const TYPE_CHECKBOX = 'checkbox';
	public const TYPE_COLLECTION = 'collection';
	public const TYPE_COUNTRY = 'country';
	public const TYPE_DATE = 'date';
	public const TYPE_FLOAT = 'float';
	public const TYPE_HREF = 'href';
	public const TYPE_IMG = 'img';
	public const TYPE_INPUT = 'input';
	public const TYPE_INT = 'int';
	public const TYPE_LANGUAGE = 'language';
	public const TYPE_LINK = 'link';
	public const TYPE_SELECT = 'select';
	public const TYPE_MULTIOBJECT = 'multiobject';
	public const TYPE_OBJECT = 'object';
	public const TYPE_SHOPCATEGORY = 'shopCategory';
	public const TYPE_SHOPVAT = 'shopVat';
	public const TYPE_TEXT = 'text';
	public const TYPE_VIDEO = 'video';
	public const DEFAULT_B = 'defaultB';
	public const DEFAULT_I = 'defaultI';
	public const DEFAULT_F = 'defaultF';
	public const DEFAULT_S = 'defaultS';
	public const DEFAULT_HTML = 'defaultHTML';
	public const DEFAULT_AI = 'defaultAI';
	public const DEFAULT_AS = 'defaultAS';
	public const DEFAULT_HREF = 'defaultHREF';
	public const DEFAULT_LINK = 'defaultLink';
	public const NAME_DIVIDER = '__';
	// these elements are not written to defaultValues
	public const GENERAL_ELEMENTS = [
		self::ELEMENT_LENGHT => [0, we_base_request::INT],
		self::ELEMENT_TYPE => ['', we_base_request::STRING],
		'name' => ['', we_base_request::STRING],
		'typeLen' => ['', we_base_request::STRING],
		'userid' => [0, we_base_request::INT],
		'usertext' => ['', we_base_request::STRING],
		'title' => ['', we_base_request::STRING],
		'desc' => ['', we_base_request::STRING],
		'keywords' => ['', we_base_request::STRING],
		'urlfield1' => ['', we_base_request::STRING],
		'urlfield2' => ['', we_base_request::STRING],
		'urlfield3' => ['', we_base_request::STRING],
		'defaultTexts' => [[], we_base_request::STRING_LIST],
		'defaultUrls' => [[], we_base_request::STRING_LIST],
		'urlfield1_len' => [0, we_base_request::INT],
		'urlfield2_len' => [0, we_base_request::INT],
		'urlfield3_len' => [0, we_base_request::INT],
	];
	public const COMMON_DEFAULTS_VALUES = [
		'editdescription' => ['', we_base_request::HTML],
		'users' => [[], we_base_request::INTLISTA],
		'required' => [false, we_base_request::BOOL],
		'uniqueID' => ['', we_base_request::STRING],
	];
	public const DEFAULTS_VALUES = [
		self::TYPE_AUDIO => [
			'defField' => self::DEFAULT_I,
			'defaults' => [
				self::DEFAULT_I => [0, we_base_request::INT],
				'defaultdir' => [0, we_base_request::INT],
				'rootdir' => [0, we_base_request::INT],
			],
		],
		self::TYPE_BINARY => [
			'defField' => self::DEFAULT_I,
			'defaults' => [
				self::DEFAULT_I => [0, we_base_request::INT],
				'defaultdir' => [0, we_base_request::INT],
				'rootdir' => [0, we_base_request::INT],
			],
		],
		self::TYPE_CHECKBOX => [
			'defField' => self::DEFAULT_B,
			'defaults' => [
				self::DEFAULT_B => [false, we_base_request::BOOL],
			],
		],
		self::TYPE_COLLECTION => [
			'defField' => self::DEFAULT_I,
			'defaults' => [
				self::DEFAULT_I => [0, we_base_request::INT],
			],
		],
		self::TYPE_COUNTRY => [
			'defField' => self::DEFAULT_S,
			'defaults' => [
				self::DEFAULT_S => ['', we_base_request::STRING],
			],
		],
		self::TYPE_DATE => [
			'defField' => self::DEFAULT_B,
			'defaults' => [
				self::DEFAULT_B => [false, we_base_request::BOOL],
				'variant' => [false, we_base_request::BOOL],
			],
		],
		self::TYPE_FLOAT => [
			'defField' => self::DEFAULT_F,
			'defaults' => [
				self::DEFAULT_F => [0, we_base_request::FLOAT],
				'variant' => [false, we_base_request::BOOL],
			],
		],
		self::TYPE_HREF => [
			'defField' => self::DEFAULT_HREF,
			'defaults' => [
				self::DEFAULT_HREF => [['int' => true, 'intID' => 0, 'extPath' => ''], 'arrayHref'],
				'hreftype' => ['all', we_base_request::STRING],
				'hrefdirectory' => [false, we_base_request::BOOL],
				'hreffile' => [false, we_base_request::BOOL],
				'variant' => [false, we_base_request::BOOL],
			],
		],
		self::TYPE_IMG => [
			'defField' => self::DEFAULT_I,
			'defaults' => [
				self::DEFAULT_I => [0, we_base_request::INT],
				'defaultThumb' => [0, we_base_request::INT],
				'defaultdir' => [0, we_base_request::INT],
				'rootdir' => [0, we_base_request::INT],
				'variant' => [false, we_base_request::BOOL],
			],
		],
		self::TYPE_INPUT => [
			'defField' => self::DEFAULT_HTML,
			'defaults' => [
				self::DEFAULT_HTML => ['', we_base_request::HTML],
				'variant' => [false, we_base_request::BOOL],
			],
		],
		self::TYPE_INT => [
			'defField' => self::DEFAULT_I,
			'defaults' => [
				self::DEFAULT_I => [0, we_base_request::INT],
				'variant' => [false, we_base_request::BOOL],
			],
		],
		self::TYPE_LANGUAGE => [
			'defField' => self::DEFAULT_S,
			'defaults' => [
				self::DEFAULT_S => ['', we_base_request::STRING],
			],
		],
		self::TYPE_LINK => [
			'defField' => self::DEFAULT_LINK,
			'defaults' => [
				self::DEFAULT_LINK => [[], 'arrayLink'],
				'variant' => [false, we_base_request::BOOL],
			],
		],
		self::TYPE_SELECT => [
			'defField' => self::DEFAULT_I,
			'defaults' => [
				self::DEFAULT_I => [0, we_base_request::INT],
				'selectKeys' => [[''], we_base_request::STRING_LIST, true],
				'selectValues' => [[''], we_base_request::STRING_LIST, true],
				'variant' => [false, we_base_request::BOOL],
			],
		],
		self::TYPE_MULTIOBJECT => [
			'defField' => self::DEFAULT_AI,
			'defaults' => [
				self::DEFAULT_AI => [[0], we_base_request::INTLISTA, true],
				'max' => [0, we_base_request::INT],
				'tableid' => [0, we_base_request::INT],
			],
			'additionals' => [// FIXME: probably obsolete
				'defaultPath' => [[''], we_base_request::STRING_LIST],
			],
		],
		self::TYPE_OBJECT => [
			'defField' => self::DEFAULT_I,
			'defaults' => [
				self::DEFAULT_I => [0, we_base_request::INT],
			],
		],
		self::TYPE_SHOPCATEGORY => [
			'defField' => self::DEFAULT_S,
			'defaults' => [
				self::DEFAULT_S => ['', we_base_request::STRING],
				'shopcatField' => ['', we_base_request::STRING],
				'shopcatShowPath' => [true, we_base_request::BOOL],
				'shopcatRootdir' => ['', we_base_request::STRING],
				'shopcatLimitChoice' => [false, we_base_request::BOOL],
			],
		],
		self::TYPE_SHOPVAT => [
			'defField' => self::DEFAULT_I,
			'defaults' => [
				self::DEFAULT_I => [0, we_base_request::INT],
			],
		],
		self::TYPE_TEXT => [
			'defField' => self::DEFAULT_HTML,
			'defaults' => [
				self::DEFAULT_HTML => ['', we_base_request::STRING],
				'wysiwyg' => [false, we_base_request::BOOL],
				'commands' => ['', we_base_request::STRING],
				'contextmenu' => ['', we_base_request::STRING],
				'bgcolor' => ['', we_base_request::STRING],
				'class' => ['', we_base_request::STRING],
				'cssClasses' => ['', we_base_request::STRING],
				'fontsizes' => ['', we_base_request::STRING],
				'formats' => ['', we_base_request::STRING],
				'tinyparams' => ['', we_base_request::STRING],
				'templates' => ['', we_base_request::STRING],
				'gallerytemplates' => ['', we_base_request::STRING],
				'removefirstparagraph' => [false, we_base_request::BOOL],
				'inlineedit' => [false, we_base_request::BOOL],
				'variant' => [false, we_base_request::BOOL],
			],
		],
		self::TYPE_VIDEO => [
			'defField' => self::DEFAULT_I,
			'defaults' => [
				self::DEFAULT_I => [0, we_base_request::INT],
				'defaultdir' => [0, we_base_request::INT],
				'rootdir' => [0, we_base_request::INT],
			],
		],
	];
	protected const URL_FIELDS = [
		'urlfield1' => 64,
		'urlfield2' => 64,
		'urlfield3' => 64,
	];

	/** @var int[] */
	protected $Workspaces = [];

	abstract function del_workspace(int $id): void;

	function add_workspace(int $id): void{
		if($id < 0){
			return;
		}
		$this->Workspaces[] = $id;
		$this->Workspaces = array_unique($this->Workspaces, SORT_NUMERIC);
	}

	public function setWorkspaces(array $ws): void{
		$this->Workspaces = array_unique(we_base_request::filterIntList($ws), SORT_NUMERIC);
	}

	/**
	 *
	 * @return int[]
	 */
	public function getWorkspaces(): array{
		return $this->Workspaces;
	}

	protected static function hrefRow(string $intID_elem_Name, int $intID, string $Path_elem_Name, string $path, string $attr, string $int_elem_Name, bool $showRadio = false, bool $int = true, bool $file = true, bool $directory = false): string{
		$checked = ($intID_elem_Name && $int) || ((!$intID_elem_Name) && (!$int));
		$weSuggest = we_gui_suggest::getInstance();
		$cmd = '';
		if($intID_elem_Name){
			// Important: all extraCmds must call we_cmd('object_selectorHref_callback') at the beginning. actually no extraCmds are used
			$cmd = 'fieldHref_selectIntHref_callback,' . ($showRadio ? 1 : 0) . ',' . $int_elem_Name;
			//add filter:''?
			$but = ( $file ?
				we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['" . $intID_elem_Name . "'].value,table:WE().consts.tables.FILE_TABLE,targetID:'" . $intID_elem_Name . "',targetName:'" . $Path_elem_Name . "',jsCmd:'" . $cmd . "',rootDirID:0,open_doc:" . (we_base_permission::hasPerm([
						we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES]) ? 0 : 1) . ",canSelectDir:" . ($directory ? 1 : 0) . "});") :
				we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:document.we_form.elements['" . $intID_elem_Name . "'].value,table:WE().consts.tables.FILE_TABLE,targetID:'" . $intID_elem_Name . "',targetName:'" . $Path_elem_Name . "',jsCmd:'" . $cmd . "'});")
				);
			$weSuggest->setAcId($int_elem_Name . we_base_file::getUniqueId())
				->setContentType(array_merge(we_base_ContentTypes::LINK_ABLE, [we_base_ContentTypes::FOLDER_DOCUMENT]))
				->setInput($Path_elem_Name, $path, ['onchange' => ($showRadio ? "document.we_form.elements['" . $int_elem_Name . "'][0].checked=true;" : '')])
				->setMaxResults(10)
				->setResult($intID_elem_Name, (string) $intID)
				->setSelector(we_gui_suggest::DocSelector)
				->setTable(FILE_TABLE)
				->setRequired(false)
			//->setJsCommandOnItemSelect($cmd)
			;
		} else {
//			$cmd = 'fieldHref_selectExtHref_callback,' . ($showRadio ? 'showRadio' : '') . ',' . $int_elem_Name;
			$but = (we_base_permission::hasPerm([we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT]) ? (
				we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('browse_server','" . $Path_elem_Name . "','" . (($directory && $file) ? 'filefolder' : ($file ? '' : we_base_ContentTypes::FOLDER_DOCUMENT)) . "',document.forms[0].elements['" . $Path_elem_Name . "'].value,'" . $cmd . "')")
				) : '');
		}

		return '<tr>' .
			($showRadio ?
			'<td>' . we_html_forms::radiobutton(($intID_elem_Name ? '1' : '0'), $checked, $int_elem_Name, g_l('weTag', (!$intID_elem_Name) ? '[href][ext]' : '[href][int]') . ':&nbsp;', true, 'defaultfont', '') . '</td><td>' :
			'<td><input type="hidden" name="' . $int_elem_Name . '" value="' . ($intID_elem_Name ? 1 : 0) . '" />'
			) .
			($intID_elem_Name ?
			$weSuggest->getHTML() :
			'<input' . ($showRadio ? ' onchange="this.form.elements[\'' . $int_elem_Name . '\'][' . ($intID_elem_Name ? 0 : 1) . '].checked=true;"' : '' ) . ' type="text" class="wetextinput" name="' . $Path_elem_Name . '" value="' . $path . '" ' . $attr . ' />'
			) . '</td>
	<td>' . $but . '</td>
	<td>' . we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $intID_elem_Name . "'].value=0;document.we_form.elements['" . $Path_elem_Name . "'].value='';_EditorFrame.setEditorIsHot(true);") . '</td>
</tr>';
	}

	public function setElementF(string $name, float $data, string $type = 'txt'): void{
		$this->setElement($name, (string) we_base_util::std_numberformat((string) $data), $type, self::ETYPE_TEXT); //@phan-suppress-current-line PhanDeprecatedFunction
	}

	public function setElementLen(string $name, $len): void{
		$this->elements[$name]['len'] = $len; //@phan-suppress-current-line PhanDeprecatedFunction
	}

	public function getElementF(string $name, float $default = 0, bool $defaultOnEmpty = false): float{
		$ret = $this->getElement($name, self::ETYPE_TEXT, $default, $defaultOnEmpty); //@phan-suppress-current-line PhanDeprecatedFunction
		return (float) $ret;
	}

	/*
	  public function getElementA(string $name, array $default = [], bool $defaultOnEmpty = false): array{
	  $ret = $this->getElement($name, self::ETYPE_TEXT, $default, $defaultOnEmpty); //@phan-suppress-current-line PhanDeprecatedFunction
	  return is_array($ret) ? $ret : $default;
	  }
	 *
	 */

	public function getElementLen(string $name): int{
		return (int) $this->getElement($name, 'len'); //@phan-suppress-current-line PhanDeprecatedFunction
	}

	protected function setElementsFromHTTP(): void{
		$dates = we_base_util::convertDateInRequest('');
		foreach($dates as $n => $v){
			'@phan-var DateTime $v';
			assert($v instanceof DateTime);
			$this->setElementI((string) $n, (int) $v->format('U'), 'date');
		}

		parent::setElementsFromHTTP();
	}

	function changeLink(string $name): void{//FIXME: can we store info in bdid? add info on type, if it is object or file?
		if(!isset($_SESSION['weS']['WE_LINK'])){
			return;
		}

		unset($_SESSION['weS']['WE_LINK']['href_int']);
		$this->setElementA($name, $_SESSION['weS']['WE_LINK'], 'link');
		unset($_SESSION['weS']['WE_LINK']);
	}

}
