<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 6cc64d37551c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-25 11:32 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_customer_EI{

	public static function getDataset(string $type, string $filename, array $arrgs = []): array{
		switch($type){
			case we_exim_ExIm::TYPE_XML:
				return self::getXMLDataset($filename, $arrgs['dataset']);
			case we_exim_ExIm::TYPE_CSV:
				return self::getCSVDataset($filename, $arrgs['delimiter'], $arrgs['enclose'], $arrgs['lineend'], $arrgs['fieldnames']);
			default:
				return [];
		}
	}

	public static function save2File(string $filename, string $code = '', string $flags = 'ab'): bool{
		try{
			return we_base_file::save($filename, $code, $flags);
		} catch (we_exception_file $e){
			return false;
		}
	}

	public static function getCustomersFieldset(): array{
		return we_customer_customer::getFieldset();
	}

	private static function getXMLDataset(string $filename, string $dataset): array{
		$xp = new we_xml_parser(WE_DOCUMENT_ROOT . $filename);
		$nodeSet = $xp->evaluate($xp->root . '/' . $dataset . '[1]/child::*');
		$nodes = /* $attrs = */[];

		foreach($nodeSet as $node){
			'@phan-var string $node';
			$nodeName = $xp->nodeName($node);
			$nodeattribs = [];
			if($xp->hasAttributes($node)){
				//$attrs += ['@n:' => g_l('modules_customer', '[none]')];
				$attributes = $xp->getAttributes($node);
				foreach($attributes as $name => $value){
					$nodeattribs[$name] = $value;
				}
			}
			$nodes[$nodeName] = $nodeattribs;
		}
		return $nodes;
	}

	private static function getCSVDataset(string $filename, string $delimiter, string $enclose, string $lineend, bool $fieldnames): array{
		$csvFile = WE_DOCUMENT_ROOT . $filename;
		if(!file_exists($csvFile) || !is_readable($csvFile)){
			return [];
		}

		$nodes = [];
		if($lineend === 'mac'){
			we_base_file::replaceInFile("\r", "\n", $csvFile);
		}
		if(!($handle = fopen($csvFile, 'r'))){
			return [];
		}
		$row = fgetcsv($handle, 4096, $delimiter === '\t' ? "\t" : $delimiter, ($enclose === '') ? '"' : $enclose);
		'@phan-var string[] $row';
		fclose($handle);
		$gl = g_l('modules_customer', '[record_field]');
		foreach($row as $no => $cur){
			if($fieldnames){
				$nodes[$cur] = [];
			} else {
				$nodes[$gl . ' ' . ($no + 1)] = [];
			}
		}

		return $nodes;
	}

	/**
	 *
	 * @return string
	 */
	public static function exportCustomersToXML(array $customers, string $filename): bool{
		$cids = array_filter($customers);
		if(!$customers){
			return false;
		}
		$db = we_database_base::$inst;
		$fields = array_diff($db->metaFieldNames(CUSTOMER_TABLE), we_customer_customer::IGNORED_EXPORT_FIELDS);
		$xml_out = we_exim_ExIm::getHeader('customer');

		foreach($db->getAllq('SELECT `' . implode('`,`', $fields) . '` FROM ' . CUSTOMER_TABLE . ' WHERE ID IN (' . implode(',', $cids) . ')') as $record){
			$customer_xml = new we_html_baseCollection('customer');
			foreach($record as $key => $data){
				'@phan-var string $key';
				[$content] = we_exim_contentProvider::formatXMLData($data);
				$customer_xml->addChild(new we_html_baseElement($key, true, [], $content));
			}
			$xml_out .= $customer_xml->getHtml() . "\n";
		}
		return self::save2File($filename, $xml_out);
	}

	/**
	 * Function creates new xml element.
	 * @param array{name:string,content:string|array,attributes?:array}[] $elements 'content':if array childs otherwise some content
	 * @return string
	 */
	static function buildXMLElement(array $elements): string{
		$out = '';
		foreach($elements as $element){
			$element = new we_html_baseElement((string) $element['name'], true, []/* $element['attributes'] */, (is_array($element['content']) ? self::buildXMLElement($element['content']) : (string) $element['content']));
			$out .= $element->getHTML();
		}
		return $out;
	}

	/**
	 * @return bool
	 */
	public static function exportCustomersToCSV(array $customers, string $filename, string $csv_enclose, string $csv_lineend, string $csv_delimiter, bool $csv_fieldnames): bool{
		$cids = array_filter($customers);
		if(!$cids){
			return false;
		}

		switch(trim($csv_lineend)){
			case g_l('modules_customer', '[unix]'):
				$lineend = "\n";
				break;
			case g_l('modules_customer', '[mac]') :
				$lineend = "\r";
				break;
			default:
				$lineend = "\r\n";
				break;
		}
		$delimiter = $csv_enclose . ($csv_delimiter === '\t' ? "\t" : trim($csv_delimiter)) . $csv_enclose;
		$db = new DB_WE();
		$fields = array_diff($db->metaFieldNames(CUSTOMER_TABLE), we_customer_customer::IGNORED_EXPORT_FIELDS);
		try{
			if($csv_fieldnames){
				we_base_file::save($filename, $csv_enclose . implode($delimiter, $fields) . $csv_enclose . $lineend);
			}

			foreach($db->getAllq('SELECT `' . implode('`,`', $fields) . '` FROM ' . CUSTOMER_TABLE . ' WHERE ID IN (' . implode(',', $cids) . ')') as $record){
				we_base_file::save($filename, $csv_enclose . implode($delimiter, $record) . $csv_enclose . $lineend, 'ab');
			}
		} catch (we_exception_file $e){
			return false;
		}
		return true;
	}

	/**
	 * @return bool
	 */
	public static function exportCustomersToODS(array $customers, string $filename): bool{
		$cids = array_filter($customers);
		if(!($cids)){
			return false;
		}
		try{
			$ods = new we_exim_ExportOpenDocument();
			$ods->addTabQuery('Customer', 'SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE ID IN (' . implode(',', $cids) . ')', we_customer_customer::IGNORED_EXPORT_FIELDS);
			$ods->getFile($filename, true);
		} catch (Exception $e){
			return false;
		}
		return file_exists($filename);
	}

	private static function getUniqueId(): string{
		return md5(uniqid(__FILE__, true));
	}

	public static function importCSV(string $csvFile, string $same, string $charset, string $csv_delimiter, string $csv_enclose, string $csv_lineend, bool $csv_fields, array $field_mappings): bool{

		if(!file_exists($csvFile) || !is_readable($csvFile)){
			return false;
		}
		$data = file($csvFile, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
		if(!$data){
			return false;
		}
		foreach($data as &$line){
			//convert charset if needed
			$line = ($charset === 'UTF-8' ? $line : (mb_convert_encoding($line, 'UTF-8', $charset . ',UTF-8') ?: ''));
			$line = str_getcsv($line, $csv_delimiter, $csv_enclose);
		}
		'@phan-var array<int,string[]> $data';
		if($csv_fields){
			$header = array_shift($data) ?: [];
		} else {
			$cnt = count($data[0]);
			$header = [];
			$gl = g_l('modules_customer', '[record_field]');
			for($i = 1; $i <= $cnt; $i++){
				$header[] = $gl . $i;
			}
		}
		foreach($data as $csv){
			self::importCustomer(array_combine($header, $csv), $field_mappings, $same);
		}
		return true;
	}

	public static function importCustomer(array $data, array $field_mappings, string $same): bool{
		$customer = new we_customer_customer();

		$fields = array_flip($field_mappings);
		$newData = [];
		foreach($data as $node_name => $node_value){
			if(isset($fields[$node_name])){
				$newData[$fields[$node_name]] = $node_value;
			}
		}
		$customer->setFromArray($newData);
		$existid = $customer->isDuplicateUsername();
		if(!$existid){
			return $customer->save();
		}

		switch($same){
			case we_exim_ExIm::COLLISION_RENAME:
				$count = 0;
				$oldname = $customer->Username;
				do{
					$count++;
					$customer->Username = $oldname . $count;
				} while($customer->isDuplicateUsername());
				return $customer->save();
			case we_exim_ExIm::COLLISION_OVERWRITE:
				return $customer->overwrite($existid);
			default:
			case we_exim_ExIm::COLLISION_SKIP:
				return false;
		}
	}

	public static function importXML(string $xmlfile, array $field_mappings, string $same): bool{
		//FIXME this does not work at the moment
		$xp = new we_xml_parser($xmlfile);

		$nodeSet = $xp->evaluate($xp->root . '/*');
		$data = [];
		foreach($nodeSet as $node){
			'@phan-var string $node';
			$node_name = $xp->nodeName($node);
			$node_value = $xp->getData($node);
			$data[$node_name] = $node_value;
		}
		return self::importCustomer($data, $field_mappings, $same);
	}

}
