<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 397c00669fde $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-18 14:09 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//@phan-file-suppress PhanUndeclaredClassMethod,PhanUndeclaredMethod,PhanUndeclaredExtendedClass
use Sabre\DAV;

class we_davfs_mainDir extends DAV\Collection{//FIXME: currently this is the same as _dir, but this should distinguish between different sources
	/** @phan-var string */
	private $path;

	private const DB = 'DB';
	private const GIT = '.git';
	private const HG = '.hg';
	private const SVN = '.svn';
	private const DIRS = [
		we_davfs_dbDir::DOC_DIR => 'DB',
		we_davfs_dbDir::TEMPLATE_DIR => 'DB',
		'weTags' => WE_CUSTOM_PATH . 'tag',
		'weTw' => WE_CUSTOM_PATH . 'tw',
		/* self::GIT => WE_CUSTOM_PATH . 'vcs/' . self::GIT,
		  self::SVN => WE_CUSTOM_PATH . 'vcs/' . self::SVN,
		  self::HG => WE_CUSTOM_PATH . 'vcs/' . self::HG */
	];

	protected static $DIRS = self::DIRS;

	public function __construct(string $path){
		$this->path = $path;
		if(we_base_WE::setDebugging()){
			self::$DIRS['webEdition'] = WEBEDITION_PATH;
		}
	}

	/**
	 * @return DAV\Node[]
	 */
	function getChildren(){
		$ret = [];
		foreach(array_keys(self::$DIRS) as $dir){
			if(!$dir || !$this->childExists($dir)){
				continue;
			}
			$ret[] = $this->getChild($dir);
		}
		return $ret;
	}

	/**
	 *
	 * @param string $name
	 * @return DAV\Node
	 * @throws DAV\Exception\NotFound
	 */
	function getChild($name){
		// We have to throw a NotFound exception if the file didn't exist
		if(!$this->childExists($name)){
			throw new DAV\Exception\NotFound('The file with name: ' . $name . ' could not be found');
		}

		switch($name){
			case we_davfs_dbDir::DOC_DIR:
				return new we_davfs_dbDir('', FILE_TABLE, $name, 0);
			case we_davfs_dbDir::TEMPLATE_DIR:
				return new we_davfs_dbDir('', TEMPLATES_TABLE, $name, 0);
			default:
				return new DAV\FSExt\Directory(self::$DIRS[$name], $name);
			/* 			case 'weTags':
			  t_e(WE_CUSTOM_PATH . 'tag');
			  return new DAV\FSExt\Directory(WE_CUSTOM_PATH . 'tag'); */
		}
	}

	/**
	 *
	 * @param string $name
	 * @return bool
	 */
	function childExists($name){
		if(!array_key_exists($name, self::$DIRS)){
			return false;
		}
		if(self::$DIRS[$name] === self::DB){
			return true;
		}
		$path = self::$DIRS[$name];
		return file_exists($path) && is_dir($path);
	}

	function getName(){
		return basename($this->path) ?: '';
	}

	/**
	 * Creates a new subdirectory.
	 *
	 * @param string $name
	 */
	public function createDirectory($name){
		// We're not allowing dots
		switch($name){
			default:
				throw new DAV\Exception\Forbidden('Permission denied to ' . $name);
			case self::HG:
			case self::GIT:
			case self::SVN:
				if(!mkdir(self::$DIRS[$name])){
					throw new DAV\Exception\Forbidden('Permission denied');
				}

				clearstatcache(true, self::$DIRS[$name]);
		}
	}

}
