<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 3d882708e829 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-09-02 17:14 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_document_textContent extends we_document_text{
	/** @var int Doc-Type of the document */
	public $DocType = 0;

	/* these fields are never read from temporary tables */
	const primaryDBFiels = 'Path,Filename,Extension,ParentID,Publish_Date,Mod_Date,CreatorID,ModifierID,Owners,RestrictOwners,WebUserID,Language,temp_template_id,DocType,TemplateID,OwnersReadOnly,temp_category,urlMap,viewType,IsProtected,Creation_Date';

	/** initializes all data not static in class and not saved by persistent slots/elements.
	 * This is called from constructor and __wakeup on serialized documents
	 */
	protected function initStatic(): void{
		parent::initStatic();
		$this->PublWhenSave = false;
		$this->IsTextContentDoc = true;

		$this->persistent_slots['DocType'] = we_base_request::INT;

		if(isWE()){
			/* if(we_modules_info::isActive(we_modules_info::SCHEDULER)){
			  array_push($this->persistent_slots, 'From', 'To');
			  } */
			$this->EditPageNrs = array_merge($this->EditPageNrs, [we_base_constants::WE_EDITPAGE_PREVIEW, we_base_constants::WE_EDITPAGE_SCHEDULER]);
		}
	}

	function editor(we_base_jsCmd $jsCmd = null): we_editor_base{
		switch($this->EditPageNr){
			case we_base_constants::WE_EDITPAGE_SCHEDULER:
				return new we_editor_scheduler($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_VALIDATION:
				return new we_editor_validateDocument($this, $jsCmd);
			default:
				return parent::editor($jsCmd);
		}
	}

	public function makeSameNew(array $keep = []): void{
		parent::makeSameNew(array_merge($keep, ['Category', 'ContentType', 'DocType', 'IsSearchable', 'Extension']));
	}

	/**
	 *
	 * @param string[] $only
	 * @param string[] $fieldTypes
	 * @return bool
	 */
	public function insertAtIndex(array $only = [], array $fieldTypes = []): bool{
		if(!($this->IsSearchable && $this->Publish_Date)){
			$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=0 AND ID=' . intval($this->ID));
			return true;
		}
		$text = '';

		if($only){
			foreach($only as $cur){
				if($this->getElementType($cur) === 'txt'){
					$dat = $this->getElementS($cur);
					//skip vars + serialized data
					if($dat && strlen($dat) > 4 && !in_array($dat[0], ['{', '[', '$'], true) && !we_base_util::isSerialized($dat)){
						$text .= ' ' . $dat;
					}
				}
			}
		} else {
			foreach($this->nextElement('txt') as $k){
				'@phan-var string $k';
				if(in_array(strpos($k, '$'), [0, 1], true)){
					//skip elements whose names are variables
					continue;
				}

				if(($dat = $this->getElementS($k)) && strlen($dat) > 4 && !in_array($dat[0], ['{', '[', '$'], true) && !we_base_util::isSerialized($dat)){
					$text .= ' ' . $dat;
				}
			}
			//variants are initialized, so nothing special to do
		}

		$text = self::formatIndexText($text);
		$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=0 AND ID=' . intval($this->ID) . ($text ? ' AND WorkspaceID!=' . intval($this->ParentID) : ''));
		if(!$text){
			return true;
		}
		return $this->DB_WE->query('REPLACE INTO ' . INDEX_TABLE . ' SET ' . we_database_base::arraySetter([
					'ID' => $this->ID,
					'WorkspaceID' => $this->ParentID,
					'Text' => $text,
					'Category' => implode('', array_filter(array_unique($this->Category))) ?: null,
					'Doctype' => intval($this->DocType),
					'Title' => self::formatIndexText($this->getElementS(self::TITLE_FIELD)) ?: null,
					'Description' => self::formatIndexText($this->getElementS(self::DESCRIPTION_FIELD)) ?: null,
					'Keywords' => self::formatIndexText($this->getElementS(self::KEYWORD_FIELD)) ?: null,
					'Language' => $this->Language ?: null,
		]));
	}

	/* publish a document */

	function getMetas(string $code): array{
		$regs = [];
		$title = (preg_match('|< ?title[^>]*>(.*)< ?/ ?title[^>]*>|i', $code, $regs) ? $regs[1] : '');
		$tempname = we_base_file::saveTemp($code);
		$metas = get_meta_tags($tempname);
		we_base_file::deleteLocalFile($tempname);
		$metas['title'] = $title;
		return $metas;
	}

	protected function formDocTypeTempl(): string{
		return
			//$this->formDocType($this->Publish_Date ? true : false) . we_html_element::br() .
			$this->formIsSearchable() .
			$this->formInGlossar();
	}

	public function we_new(): void{
		parent::we_new();
		$this->Filename = $this->getDefaultFilename();
	}

	protected function we_load(int $from = self::LOAD_MAIN_DB): bool{
		$found = false;
		switch($from){
			case self::LOAD_TEMP_DB:
				$found = parent::we_load($from);
				if($found){
					if($this instanceof we_document_webEdition){
						$this->Category = $this->temp_category;
						$this->TemplateID = $this->temp_template_id;
					}
					$this->OldPath = $this->Path;
					break;
				}
				//load main-data
				$from = self::LOAD_MAIN_DB;
			//no break;
			case self::LOAD_MAIN_DB:
				$found = parent::we_load($from);
				break;
		}
		$this->OldPath = $this->Path;
		return $found;
	}

	public function we_save(bool $resave = false, bool $skipHook = false): bool{
		if(!$skipHook){
			$hook = new we_hook_handler('preSave', ['obj' => $this, 'resave' => $resave]);
			//check if doc should be saved
			if(!$hook->executeHook()){
				$this->errMsg = $hook->getErrorString();
				return false;
			}

			//check for warnings and notices
			if(($tmp = $hook->getWarningString())){
				$this->warningMsg[] = $tmp;
			}
			if(($tmp = $hook->getNoticeString())){
				$this->noticeMsg[] = $tmp;
			}
		}

		if((!$this->ID) && !we_contents_root::we_save(false)){ // when no ID, then allways save before in main table
			return false;
		}
		if(!$resave){
			$this->ModifierID = !we_base_WE::$schedulerActive ? we_users_user::getID() : 0;
			$this->Mod_Date = new DateTime();
			$this->wasUpdate = true;
			we_base_history::insertIntoHistory($this);
			$this->saveDocumentCustomerFilter();
		}

		// allways store in temp-table
		$ret = $this->saveTmp(!$resave);
		$this->OldPath = $this->Path;

		if($this->isVersioned()){
			we_versions_version::save($this);
		}

		/* hook */
		if(!$skipHook){
			$hook = new we_hook_handler('save', ['obj' => $this, 'resave' => $resave]);
			//check if doc should be saved
			if(!$hook->executeHook()){
				$this->errMsg = $hook->getErrorString();
				return false;
			}

			//check for warnings and notices
			if(($tmp = $hook->getWarningString())){
				$this->warningMsg[] = $tmp;
			}
			if(($tmp = $hook->getNoticeString())){
				$this->noticeMsg[] = $tmp;
			}
		}

		return $ret;
	}

	public function we_publish(bool $DoNotMark = false, bool $saveinMainDB = true, bool $skipHook = false): bool{
		if(!$skipHook){
			$hook = new we_hook_handler('prePublish', [$this]);
			//check if doc should be saved
			if(!$hook->executeHook()){
				$this->errMsg = $hook->getErrorString();
				return false;
			}

			//check for warnings and notices
			if(($tmp = $hook->getWarningString())){
				$this->warningMsg[] = $tmp;
			}
			if(($tmp = $hook->getNoticeString())){
				$this->noticeMsg[] = $tmp;
			}
		}
		$this->oldCategory = we_base_request::filterIntList(fS('SELECT Category FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID=' . intval($this->ID)));

		if(!($saveinMainDB ? we_contents_root::we_save(true) : $this->we_save($DoNotMark))){
			return false; // calls the root function, so the document will be saved in main-db but it will not be written!
		}

		$oldPublished = $this->Publish_Date;

		$this->Publish_Date = new DateTime();

		if(!$this->writeDocWhenPubl()){
			$this->Publish_Date = $oldPublished;
			return false;
		}

		if(!$DoNotMark){
			if(!$this->DB_WE->query('UPDATE ' . $this->DB_WE->escape($this->Table) . ' SET ' . we_database_base::arraySetter(['Publish_Date' => $this->Publish_Date]) . ' WHERE ID=' . intval($this->ID))){
				return false; // mark the document as published;
			}
		}

		//Bug #5505
//		if($oldPublished == 0 || $this->isMoved() || $this->Category != $this->oldCategory || $oldDocType != $this->DocType){
		//FIXME: changes of customerFilter are missing here
		$this->rewriteNavigation();
		//	}
		if(!empty($_SESSION['weS']['versions']['fromScheduler']) && ($this->isVersioned())){
			we_versions_version::save($this, 'published');
		}
		/* hook */
		if(!$skipHook){
			$hook = new we_hook_handler('publish', ['obj' => $this, 'prePublishTime' => $oldPublished]);
			//check if doc should be saved
			if(!$hook->executeHook()){
				$this->errMsg = $hook->getErrorString();
				return false;
			}

			//check for warnings and notices
			if(($tmp = $hook->getWarningString())){
				$this->warningMsg[] = $tmp;
			}
			if(($tmp = $hook->getNoticeString())){
				$this->noticeMsg[] = $tmp;
			}
		}
		we_base_delete::deleteTemporaryContent($this->ID, $this->Table);
		return $this->insertAtIndex();
	}

	public function we_unpublish(bool $skipHook = false): bool{
		if(!$this->ID || (file_exists($this->getRealPath(true)) && !we_base_file::deleteLocalFile($this->getRealPath(!$this->isMoved())))){
			return false;
		}
		if(!$this->DB_WE->query('UPDATE ' . $this->DB_WE->escape($this->Table) . ' SET Publish_Date=null WHERE ID=' . intval($this->ID))){
			return false;
		}

		$this->Publish_Date = null;
		$this->rewriteNavigation();

		/* version */
		if($this->isVersioned()){
			we_versions_version::save($this, 'unpublished');
		}
		/* hook */
		if(!$skipHook){
			$hook = new we_hook_handler('unpublish', [$this]);
			//check if doc should be saved
			if(!$hook->executeHook()){
				$this->errMsg = $hook->getErrorString();
				return false;
			}

			//check for warnings and notices
			if(($tmp = $hook->getWarningString())){
				$this->warningMsg[] = $tmp;
			}
			if(($tmp = $hook->getNoticeString())){
				$this->noticeMsg[] = $tmp;
			}
		}

		$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=0 AND ID=' . intval($this->ID));

		return true;
	}

	public function we_republish(bool $rebuildMain = true): bool{
		return ($this->Publish_Date ?
			$this->we_publish(true, $rebuildMain) :
			$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=0 AND ID=' . intval($this->ID))
			);
	}

	function ModifyPathInformation(int $parentID): void{
		$this->setParentID($parentID);
		$this->wasUpdate = true;
		$this->savePersistentSlotsToDB(['Path', 'ParentID']);
		$this->insertAtIndex();
		$this->modifyChildrenPath(); // only on folders, because on other classes this function is empty
	}

	private function saveTmp(bool $write = true): bool{
		if(!$this->saveContentDataInDB(TEMPORARY_DOCTARGET)){
			return false;
		}
		if(!$this->savePersistentSlotsToDB(['Path', 'Filename', 'Extension', 'ParentID', 'CreatorID', 'ModifierID', 'RestrictOwners', 'Owners', 'Publish_Date', 'Mod_Date',
				'temp_template_id', 'temp_category', 'DocType', 'WebUserID', 'Language'])){
			return false;
		}
		return ($write ? $this->writeDocument() : true);
	}

	protected function writeMainDir(string $content): bool{
		return true; // do nothing!
	}

	private function writeDocWhenPubl(): bool{
		if(!$this->ID){
			return false;
		}
		$realPath = $this->getRealPath();
		$parent = str_replace('\\', '/', dirname($realPath));
		$cf = [];
		while(!we_base_file::checkAndMakeFolder($parent, true)){
			$cf[] = $parent;
			$parent = str_replace('\\', '/', dirname($parent));
		}
		try{
			for($i = (count($cf) - 1); $i >= 0; $i--){
				we_base_file::createLocalFolderByPath($cf[$i]);
			}
			$doc = $this->getDocumentToSave();
		} catch (Throwable $e){
			t_e($e->getMessage());
			return false;
		}
		return parent::writeMainDir($doc);
	}

	public function revert_published(): void{
		we_base_delete::deleteTemporaryContent($this->ID, $this->Table);
		$this->initByID($this->ID);
		$this->Mod_Date = $this->Publish_Date ?: new DateTime();
		$this->we_save();
		$this->we_publish();
		if(we_modules_info::isActive(we_modules_info::WORKFLOW) && $this->ContentType === we_base_ContentTypes::WEDOCUMENT){
			if(we_workflow_utility::inWorkflow($this->ID, $this->Table)){
				we_workflow_utility::removeDocFromWorkflow($this->ID, $this->Table, we_users_user::getID(), '');
			}
		}
	}

	function publInScheduleTable(): ?DateTime{
		if(we_modules_info::isActive(we_modules_info::SCHEDULER)){
			return we_schedule_scheduler::publInScheduleTable($this->ID, $this->Table, $this->schedArr, $this->DB_WE);
		}
		return null;
	}

	function checkPathDiffAndCreate(): bool{
		$pathByID = $this->getParentPath();
		if(!in_array($this->ParentPath, [$pathByID, '', '/'])){
			if(!$this->DocType){
				return false;
			}
			$doctype = new we_doctype_docTypes($this->DocType);
			if(!$doctype->SubDir){
				return false;
			}
			$tail = '';
			switch($doctype->SubDir){
				case self::SUB_DIR_YEAR:
					$tail = date('Y');
					break;
				case self::SUB_DIR_YEAR_MONTH:
					$tail = date('Y') . '/' . date('m');
					break;
				case self::SUB_DIR_YEAR_MONTH_DAY:
					$tail = date('Y') . '/' . date('m') . '/' . date('d');
					break;
			}
			if(rtrim($pathByID, '/') . '/' . $tail != $this->ParentPath){
				return false;
			}

			$this->ParentID = $this->getParentIDFromParentPath();
			$this->Path = $this->getPath();
		}

		return parent::checkPathDiffAndCreate();
	}

}
