<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 14778142e856 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-08-25 15:19 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
we_modules_info::isActive(we_modules_info::EXPORT);

class we_export_view extends we_modules_view{
	/** @var we_export_export */
	var $Model;

	public function __construct(){
		parent::__construct(we_modules_info::EXPORT);
		$this->Model = new we_export_export();
	}

	function getCommonHiddens(array $cmds = []): string{
		return
			parent::getCommonHiddens($cmds) .
			we_html_element::hiddens([
				'table' => we_base_request::_(we_base_request::TABLE, 'table', FILE_TABLE),
				'id' => (string) $this->Model->ID,
				'IsFolder' => (string) $this->Model->IsFolder,
				'selDocs' => implode(',', $this->Model->selDocs),
				'selTempl' => implode(',', $this->Model->selTempl),
				'selObjs' => implode(',', $this->Model->selObjs),
				'selClasses' => implode(',', $this->Model->selClasses),
				'selDocs_open' => implode(',', we_base_request::intList('selDocs_open', [])),
				'selTempl_open' => implode(',', we_base_request::intList('selTempl_open', [])),
				'selObjs_open' => implode(',', we_base_request::intList('selObjs_open', [])),
				'selClasses_open' => implode(',', we_base_request::intList('selClasses_open', [])),
		]);
	}

	function getJSTop(): string{
		return parent::getJSTop() .
			we_html_element::jsScript(WE_JS_MODULES_DIR . '/export/export_top.js', "parent.document.title='" . $this->getModTitle() . "'");
	}

	function getJSProperty(array $jsVars = []): string{
		$arr = [
			FILE_TABLE => 'selDocs',
			TEMPLATES_TABLE => 'selTempl'
		];
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$arr[OBJECT_FILES_TABLE] = 'selObjs';
			$arr[OBJECT_TABLE] = 'selClasses';
		}

		$dynVars = [
			'initialTreeData' => [
				'selectedItems' => [],
				'openFolders' => []
			],
			'modelProperties' => [
				'isFolder' => intval($this->Model->IsFolder),
				'currentTable' => we_base_request::_(we_base_request::TABLE, "table", FILE_TABLE)
			]
		];

		foreach($arr as $table => $elem){
			$dynVars['initialTreeData']['selectedItems'][$table] = $this->Model->$elem;
			$dynVars['initialTreeData']['openFolders'][$table] = we_base_request::intList($elem . '_open');
		}

		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'export/export_prop.js', '', ['id' => 'loadVarExport_prop', 'data-dynVars' => setDynamicVar($dynVars)]);
	}

	function processCommands(we_base_jsCmd $jscmd): void{
		switch(we_base_request::string('cmd')){
			case 'we_export_new':
				if(!we_base_permission::hasPerm([we_base_perm::EXIM_EXPORT_NEW])){
					$jscmd->addMsg(g_l('export', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				$this->Model = new we_export_export();
				$jscmd->addCmd('loadHeaderFooter', $this->Model->Text);

				break;
			case 'we_export_new_group':
				if(!we_base_permission::hasPerm([we_base_perm::EXIM_EXPORT_NEW])){
					$jscmd->addMsg(g_l('export', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				$this->Model = new we_export_export();
				$this->Model->Text = g_l('export', '[newFolder]');
				$this->Model->IsFolder = true;
				$jscmd->addCmd('loadHeaderFooter', $this->Model->Text);
				break;
			case 'we_export_edit':
				if(!we_base_permission::hasPerm([we_base_perm::EXIM_EXPORT_EDIT])){
					$jscmd->addMsg(g_l('export', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				$this->Model = new we_export_export(we_base_request::int("cmdid"));
				$jscmd->addCmd('loadHeaderFooter', $this->Model->Text);

				break;
			case 'we_export_save':
				if(!we_base_permission::hasPerm([we_base_perm::EXIM_EXPORT_NEW])){
					$jscmd->addMsg(g_l('export', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				if(we_export_export::filenameNotValid($this->Model->Text)){
					$jscmd->addMsg(g_l('export', '[wrongtext]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				// check if filename is valid.
				if(we_export_export::exportToFilenameValid($this->Model->Filename)){
					$jscmd->addMsg(g_l('export', '[wrongfilename]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				if(!trim($this->Model->Text)){
					$jscmd->addMsg(g_l('export', '[name_empty]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				$oldpath = $this->Model->Path;
				// set the path and check it
				$this->Model->setPath();
				if($this->Model->pathExists($this->Model->Path)){
					$jscmd->addMsg(g_l('export', '[name_exists]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				if($this->Model->isSelf()){
					$jscmd->addMsg(g_l('export', '[path_nok]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				if($this->Model->ParentID){
					$folder = fB('SELECT IsFolder FROM ' . EXPORT_TABLE . ' WHERE ID=' . intval($this->Model->ParentID));
					if(!$folder){
						$jscmd->addMsg(g_l('export', '[path_nok]'), we_base_util::WE_MESSAGE_ERROR);
						break;
					}
				}
				if(!empty($this->Model->Folder) && $this->Model->ParentID){
					$folder = fB('SELECT IsFolder FROM ' . FILE_TABLE . ' WHERE ID=' . intval($this->Model->Folder));
					if(!$folder){
						$jscmd->addMsg(g_l('export', '[path_nok]'), we_base_util::WE_MESSAGE_ERROR);
						break;
					}
				}

				$newone = true;
				if($this->Model->ID){
					$newone = false;
				}

				$this->Model->save();

				if($this->Model->IsFolder && $oldpath != '' && $oldpath != '/' && $oldpath != $this->Model->Path){
					foreach($this->db->getAllq('SELECT ID FROM ' . EXPORT_TABLE . ' WHERE Path LIKE "' . $this->db->escape($oldpath) . '%" AND ID!=' . intval($this->Model->ID), true) as $id){
						$this->db->query('UPDATE ' . EXPORT_TABLE . ' SET Path="' . $this->Model->evalPath((int) ($id), EXPORT_TABLE) . '" WHERE ID=' . $id);
					}
				}

				if($newone){
					$jscmd->addCmd('makeTreeEntry', [
						'id' => $this->Model->ID,
						'parentid' => $this->Model->ParentID,
						'text' => $this->Model->Text,
						'open' => false,
						'contenttype' => ($this->Model->IsFolder ? we_base_ContentTypes::FOLDER_COMMON : 'we/export'),
						'table' => EXPORT_TABLE
					]);
					$jscmd->addCmd('drawTree');
				} else {
					$jscmd->addCmd('updateTreeEntry', [
						'id' => $this->Model->ID,
						'parentid' => $this->Model->ParentID,
						'text' => $this->Model->Text,
					]);
				}
				$jscmd->addMsg(g_l('export', ($this->Model->IsFolder == 1 ? '[save_group_ok]' : '[save_ok]')), we_base_util::WE_MESSAGE_NOTICE);
				$jscmd->addCmd('loadHeaderFooter', $this->Model->Text);
				break;
			case 'we_export_delete':
				if(!we_base_permission::hasPerm([we_base_perm::EXIM_EXPORT_DELETE])){
					$jscmd->addMsg(g_l('export', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}

				if($this->Model->delete()){
					$jscmd->addCmd('deleteTreeEntry', $this->Model->ID);
					$jscmd->addMsg(g_l('export', ($this->Model->IsFolder ? '[delete_group_ok]' : '[delete_ok]')), we_base_util::WE_MESSAGE_NOTICE);
					$jscmd->addCmd('home');
					$this->Model = new we_export_export();
					break;
				}
				$jscmd->addMsg(g_l('export', ($this->Model->IsFolder == 1 ? '[delete_group_nok]' : '[delete_nok]')), we_base_util::WE_MESSAGE_ERROR);

				break;
			case 'start_export':
				we_exim_Export::unsetPreserves(); // reset exporter
				$_REQUEST['cmd'] = 'do_export';
				switch($this->Model->ExportTo){
					case 'local':
						$this->Model->ExportFilename = TEMP_PATH . $this->Model->Text;
						we_base_file::insertIntoCleanUp($this->Model->ExportFilename, 24 * 3600);
						break;
					default:
						$this->Model->ExportFilename = WE_DOCUMENT_ROOT . $this->Model->ServerPath . '/' . $this->Model->Text;
				}
				break;
			default:
		}

		$_SESSION['weS']['module_sessions']['export'] = $this->Model;
	}

	function processVariables(): void{//FIXME use table datatypes or make like in we_export_wizard::getExportVars()
		if(isset($_SESSION['weS']['module_sessions']['export'])){
			$this->Model = $_SESSION['weS']['module_sessions']['export'];
		}

		if(isset($_SESSION['weS']['module_sessions']['exportVars'])){
			unset($_SESSION['weS']['module_sessions']['exportVars']);
		}
		foreach($this->Model->persistent_slots as $varname => $type){
			$this->Model->{$varname} = we_base_request::_($type, $varname, $this->Model->{$varname});
		}

		$this->page = we_base_request::int('page', $this->page);
	}

	public function getHomeScreen(): string{
		$hiddens = ['cmd' => 'home'];
		$content = we_html_button::create_button('new_export', "javascript:we_cmd('we_export_new');", '', '', !we_base_permission::hasPerm([we_base_perm::EXIM_EXPORT_NEW])) . '<br/>' .
			we_html_button::create_button('new_export_group', "javascript:we_cmd('we_export_new_group');", '', '', !we_base_permission::hasPerm([we_base_perm::EXIM_EXPORT_NEW]));
		return parent::getActualHomeScreen('export', $content, we_html_element::form(['name' => we_base_constants::WE_FORM], $this->getCommonHiddens($hiddens) . we_html_element::hidden('home', 0)
		));
	}

}
