<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: c79dcfff246d $
 * $Author: Lukas Imhof $
 * $Date: 2020-10-07 14:03 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_fileupload_resp_multiimport extends we_fileupload_resp_import{

	public function processRequest(): array{
		// manage filenumber
		if($this->controlVars['formcount']){
			return parent::processRequest();
		}
		return [];
	}

	protected function checkSetParentId(): void{
		$curParentID = $this->fileVars['parentID'] ?: 0;
		$curParentPath = $origParentPath = $this->fileVars['parentID'] ? id2path($this->fileVars['parentID']) : '';

		$this->fileVars['weFilePath'] = trim($this->fileVars['weFilePath']);
		if($this->docVars['importDirs'] && $this->fileVars['weFilePath']){
			foreach(explode('/', $this->fileVars['weFilePath']) as $dir){
				if(($newParentID = path_to_id(($newParentPath = $curParentPath . '/' . $dir)))){
					$curParentID = $newParentID;
					$curParentPath = $newParentPath;
				} else {
					if(!we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_FOLDER_NEW])){
							break;
					}
					$curParentPath = $newParentPath;
					$folder = new we_folder_document();
					$folder->initByPath($curParentPath);
					$folder->we_save();
					$curParentID = $folder->ID;
				}
			}
		}
		$this->fileVars['parentID'] = $curParentID;
		$this->fileVars['innerPath'] = ($inner = trim(str_replace($origParentPath, '', $curParentPath), '/')) ? $inner . '/' : '';
	}

	protected function postprocess(): array{
		$response = parent::postprocess();

		if($response['status'] === 'failure'){
			if(!isset($_SESSION['weS']['WE_IMPORT_FILES_ERRORs'])){
				$_SESSION['weS']['WE_IMPORT_FILES_ERRORs'] = [];
			}
			$_SESSION['weS']['WE_IMPORT_FILES_ERRORs'][] = ['filename' => $this->fileVars['weFileName'], 'error' => $response['message']];
		} else {
			$_SESSION['weS']['WE_IMPORT_FILES_SUCCESS_IDS'] = $_SESSION['weS']['WE_IMPORT_FILES_SUCCESS_IDS'] ?? [];
			$_SESSION['weS']['WE_IMPORT_FILES_DOCUMENTS'] = $_SESSION['weS']['WE_IMPORT_FILES_DOCUMENTS'] ?? [];

			$_SESSION['weS']['WE_IMPORT_FILES_SUCCESS_IDS'][] = $response['weDoc']['id'];
			$_SESSION['weS']['WE_IMPORT_FILES_DOCUMENTS'][] = $response['weDoc'];
		}

		if($this->controlVars['formnum'] === $this->controlVars['formcount']){
			$response['success'] = $_SESSION['weS']['WE_IMPORT_FILES_SUCCESS_IDS'] ?? [];
			$response['imported_files'] = $_SESSION['weS']['WE_IMPORT_FILES_DOCUMENTS'] ?? [];

			if(isset($_SESSION['weS']['WE_IMPORT_FILES_ERRORs']) && $this->controlVars['formnum'] !== 0){
				$filelist = '';
				foreach($_SESSION['weS']['WE_IMPORT_FILES_ERRORs'] as $err){
					$filelist .= '- ' . $err['filename'] . ' => ' . $err['error'] . '\n';
				}
				unset($_SESSION['weS']['WE_IMPORT_FILES_ERRORs']);
				$response['completed'] = ['message' => sprintf(g_l('importFiles', '[error][general]'), $filelist), 'type' => we_base_util::WE_MESSAGE_ERROR];
			} else {
				$completed = g_l('importFiles', '[finished]');
				if($response['imported_files']){
					$completed .= '</br>';
					foreach($response['imported_files'] as $file){
						$completed .= '</br>- ' . $file['innerPath'] . $file['text'];
					}
				}
				$response['completed'] = ['message' => $completed, 'type' => we_base_util::WE_MESSAGE_NOTICE];
			}

			unset($_SESSION['weS']['WE_IMPORT_FILES_SUCCESS_IDS']);
			unset($_SESSION['weS']['WE_IMPORT_FILES_DOCUMENTS']);
		}

		return $response;
	}

}
