<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 700f8a85f116 $
 * $Author: Lukas Imhof $
 * $Date: 2020-10-08 21:44 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_fileupload_ui_importer extends we_fileupload_ui_base{ // FIXME: extend ui_preview

	public function __construct(string $name, array $acceptedCT = [], array $acceptedExt = [], array $forbiddenCT = [], array $forbiddenExt = []){
		$this->noPermsCT = $this->getForbiddenCtsFromPerms();

		parent::__construct('we_File', $acceptedCT, $acceptedExt, $forbiddenCT, $forbiddenExt);
		$this->responseClass = 'we_fileupload_resp_multiimport';

		$this->type = 'importer';
		$this->isMultiUpload = true;
		$this->doCommitFile = true;

		$this->isMultiselect = true;

		$this->controls = [
			'showControls' => self::CONTROLS_SHOW_TRUE,
			'showControlsInitialState' => self::CONTROLS_SHOW_TRUE,
			'showBtnsUploadCancel' => self::CONTROLS_SHOW_FALSE,
			'showEditOptions' => self::CONTROLS_SHOW_TRUE,
			'showImportMeta' => self::CONTROLS_SHOW_FALSE
		];

		$this->itemControlls['showDelete'] = self::CONTROLS_SHOW_TRUE;
	}

	public function getHTML(string $hiddens = ''): string{
		$topParts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine('', $this->getFormUploader($this->getDropZoneContent(), false)));

		$uploaderBusy = we_html_element::div(['class' => 'containerUploaderBusy'], we_html_element::div(['class' => 'elemSpinner'], '<i class="fa fa-2x fa-spinner fa-pulse"></i>') .
				we_html_element::div(['class' => 'elemText'])
		);

		$content = we_html_element::hiddens(['we_cmd[0]' => 'import_files', 'cmd' => 'content']) .
			we_html_element::div(['class' => 'fileuploadImporter'], we_html_element::form(['action' => '',
					'name' => we_base_constants::WE_FORM,
					'enctype' => 'multipart/form-data'
					], $hiddens .
					we_html_element::div(['class' => 'dialogboxTop'], (new we_html_multiIconBox($topParts, 'selectFiles'))->marginLeft()->headline(g_l('importFiles', '[step2]'))->get()) .
					we_html_element::div(['id' => 'div_upload_files', 'class' => 'importer_containerUploadFiles'], self::getHtmlLoup() . we_html_element::div(['class' => 'uploadFiles',
							'id' => 'uploadFiles']))
				) . we_html_element::div(['class' => 'uploaderMask hidden']) . $uploaderBusy
		);

		return we_html_element::body(['class' => 'weDialogBody'], $content);
	}

	public function getButtonWrapped(string $type, bool $disabled = false, string $prefix = '_btn'): string{ // FIXME: this stuff is obsolete too...
		switch($type){
			case 'browse':
				$isIE10 = we_base_browserDetect::isIE() && we_base_browserDetect::getIEVersion() < 11;

				$fileInputFiles = we_html_element::input(array_filter(['class' => 'fileInput fileInputHidden' . ($isIE10 ? ' fileInputIE10' : ''),
						'style' => 'width: 170px;',
						'type' => 'file',
						'name' => $this->name,
						'multiple' => 'multiple',
						'webkitdirectory' => '',
						'id' => $this->name,
						'accept' => trim($this->typeCondition['accepted']['all'], ','),
				]));
				$btnFiles = we_html_button::create_button('fat:browse_harddisk,fa-lg fa-file-o', 'javascript:void(0)', '', '', $disabled, false, $prefix, false, '', 'weBtn noMarginLeft');

				$fileInputDirs = we_html_element::input(array_filter(['class' => 'fileInput fileInputHidden' . ($isIE10 ? ' fileInputIE10' : ''),
						'style' => 'width: 170px;',
						'type' => 'file',
						'name' => $this->name . '_dirs',
						'multiple' => 'multiple',
						'webkitdirectory' => 'webkitdirectory',
						'id' => $this->name . '_dirs',
						'accept' => trim($this->typeCondition['accepted']['all'], ','),
				]));
				$btnDirs = we_html_button::create_button('fat:browse_harddisk,fa-lg fa-folder-o', 'javascript:void(0)', '', '', $disabled, false, $prefix, false, '', 'weBtn noMarginLeft');

				return we_html_element::div(['id' => 'div_' . $this->name . '_fileInputWrapper', 'class' => 'we_fileInputWrapper', 'style' => 'vertical-align:top;display:inline-block;'], $fileInputFiles . $btnFiles) .
					/*we_html_element::div(['id' => 'div_' . $this->name . '_fileInputWrapper', 'class' => 'we_fileInputWrapper', 'style' => 'vertical-align:top;display:inline-block;'], $fileInputDirs . $btnDirs)*/''; // IMI FILEUPLOAD
			default:
				return parent::getButtonWrapped($type, $disabled, $prefix);
		}
	}

	public function getJs(): string{
		return self::getJSFileuploader() . self::getJSUtils() . $this->getJSInit();
	}

	//TODO: add param filetype
	public static function getBtnImportFiles(int $parentID = 0, bool $parentFixed = false, string $cmdUploadFinished = '', string $text = ''): string{
		return we_html_button::create_button('fa:' . ($text ?: 'btn_import_files') . ',fa-lg fa-upload', "javascript:we_cmd('import_files','" . $parentID . "', " . ($parentFixed ? 1 : 0) . ", '" . $cmdUploadFinished . "')");
	}

}
