<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: f389fd111aab $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-02-17 01:17 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * @abstract implementation class of metadata reader for IPTC data
 * @author Alexander Lindenstruth
 * @since 5.1.0.0 - 27.09.2007
 * @uses IPTC PEAR_IPTC Package for reading IPTC data. See link below for more information
 * @link http://pear.php.net/package/Image_IPTC/ PEAR IPTC Package
 */
class we_metadata_IPTC extends we_metadata_metaData{
	/** @var string[] */
	const usedFields = ['byline_title', 'byline', 'caption_writer', 'caption', 'category', 'city', 'copyright_string', 'country_code', 'country', 'created_date', 'credit',
		'edit_status', 'fixture_identifier', 'headline', 'keywords', 'local_caption', 'object_cycle', 'object_name', 'original_transmission_reference', 'originating_program',
		'priority', 'program_version', 'province_state', 'reference_date', 'reference_number', 'reference_service', 'release_date', 'release_time', 'source', 'special_instructions',
		'supplementary_category'];

	/** @var string[] */
	var $accesstypes = ['read'];

	public function __construct(string $filetype){
		$this->filetype = $filetype;
		$this->accesstypes = ['read'];
	}

	/**
	 *
	 * @param string[] $selection
	 * @return string[]
	 */
	protected function getInstMetaData(array $selection = []): array{
		if(!$this->valid){
			return [];
		}

		// seems not to work correctly so only an empty array is returned to caller:
		$this->metadata = [];
		//$this->metadata = array("Copyright" => "/me","Make" => "Fuji");

		$iptcData = new Image_IPTC($this->datasource);
		if(!$iptcData->isValid()){
			return [];
		}
		if($selection){
			// fetch some tags
			foreach($selection as $value){
				$this->metadata[] = $iptcData->getTag($value);
			}
		} else {
			foreach(self::usedFields as $fieldName){
				$data = $iptcData->getTag($fieldName);
				if($data !== null){
					$this->metadata[$fieldName] = $data;
				}
			}
		}

		return $this->metadata;
	}

}
