<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 14778142e856 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-08-25 15:19 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_modules_view implements we_modules_viewIF{
	/** @var we_database_base */
	protected $db;

	/** @var we_base_mainmodel */
	var $Model;

	/** @var int */
	protected $page = 0;

	/** @var string */
	protected $moduleType = '';

	public function __construct(string $modelType){
		$this->db = new DB_WE();
	}

	function getCommonHiddens(array $cmds = []): string{
		return we_html_element::hiddens([
				'cmd' => ($cmds['cmd'] ?? ''),
				'cmdid' => ($cmds['cmdid'] ?? ''),
				'pnt' => ($cmds['pnt'] ?? ''),
				'tabnr' => ($cmds['tabnr'] ?? '')
		]);
	}

	protected function getModTitle(): string{
		$modData = we_modules_info::getModuleField($this->moduleType, 'text');
		return ($modData ? 'webEdition ' . g_l('global', '[modules]') . ' - ' . $modData : '');
	}

	function getJSTop(): string{
		return '';
	}

	function getJSProperty(array $jsVars = []): string{
		return '';
	}

	public function processCommands(we_base_jsCmd $jscmd): void{
		switch(we_base_request::string('cmd', '')){
			case 'switchPage':
				break;
			default:
		}
	}

	public function isSaved(): bool{
		assert($this->Model instanceof we_base_model);
		return $this->Model->ID > 0;
	}

	public function processVariables(): void{
		$this->page = we_base_request::int('page', $this->page);

		//FIXME: we should use persistent slots to set all request vars! like we do in we_contents_base::setPersistentSlot
	}

	public function getPage(): string{
		return (string) ($this->page);
	}

	public function getActualHomeScreen(string $mod, string $content, string $body = ''): string{
		$row = 0;
		$starttable = (new we_html_table(['cellpadding' => 7], 3, 1))
			->setCol($row++, 0, we_modules_info::getModuleField($mod, 'text'), ['class' => 'defaultfont titleline', 'colspan' => '3'])
			->setColAttributes($row++, 0, ['class' => 'defaultfont', 'colspan' => '3'])
			->setCol($row++, 0, $content, ['style' => 'text-align:center']);

		return we_html_tools::getHtmlTop('', we_html_element::cssLink(CSS_DIR . 'tools_home.css') . $this->getJSProperty(), we_html_element::body([
					'class' => 'home',
					'onload' => "loaded = true;var we_is_home = 1;"
					], we_html_element::div(['id' => 'tabelle'], $starttable->getHtml()) .
					we_html_element::div(['id' => 'modimage'], we_html_element::img([
							'src' => IMAGE_DIR . 'bg-editor.png',
							'class' => 'blank_editor_logo'
						])
					) .
					$body)
		);
	}

	protected function formInput(string $name, string $text = '', int $width = 0, int $maxlength = 0, array $attribs = []): string{
		assert($this->Model instanceof we_base_model);
		[$realName] = explode('#', $name);
		$ps = $this->Model->{$realName};
		$data = (isset($GLOBALS['meta'][$realName]) ? $GLOBALS['meta'][$realName]['default'] : $ps);

		return we_html_element::div(['style' => 'position:relative'], ($text ? $text .
				we_html_element::br() : '') .
				we_html_tools::textInput($name, (string) ($data ?? ''), $maxlength, $attribs, $attribs['type'] ?? 'txt', $width ? $width . 'px' : '0'));
	}

}
