<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: c2f64ea9c91f $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-12-04 00:30 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_rpcCmd_documentSetPropertyOrElement extends we_rpc_cmd{

	function execute(): we_rpc_response{

		$id = we_base_request::int('we_cmd', 0, 'id');
		$table = we_base_request::_(we_base_request::TABLE, 'we_cmd', '', 'table');
		$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', '', 'transaction');
		$name = we_base_request::string('we_cmd', '', 'name');
		$type = we_base_request::string('we_cmd', '', 'type');
		$key = we_base_request::string('we_cmd', we_contents_base::ETYPE_TEXT, 'key');
		$value = we_base_request::string('we_cmd', '', 'value');

		if(!$table || !($id || $we_transaction)){
			$this->resp->setData('error', ['cannot load wedoc']);
			return $this->resp;
		}
		if(!$name){
			$this->resp->setData('error', ['no field name defined']);
			return $this->resp;
		}

		switch($table){
			case FILE_TABLE:
				$we_doc = $we_transaction ? we_contents_root::getSessionDocument($we_transaction) : new we_document_webEdition();
				break;
			case OBJECT_FILES_TABLE:
			case VFILE_TABLE:
			default:
				$this->resp->setData('error', ['not implemented']);
				return $this->resp;
		}

		$isFromSessDat = false;
		if($we_transaction){
			$isFromSessDat = true;
		} elseif($id){
			$we_doc->initByID($id);
		} else {
			$this->resp->setData('error', ['cannot load wedoc']);
			return $this->resp;
		}
		switch($key){
			case we_contents_base::ETYPE_TEXT:
				$we_doc->setElementS($name, $value, $type);
				break;
			case we_contents_base::ETYPE_NUMBER:
				$we_doc->setElementI($name, (int) $value, $type);
				break;
		}
		$ret = ($isFromSessDat ?
			$we_doc->saveInSession($we_transaction) :
			$we_doc->save(false, true));

		$this->resp->setData('data', ($ret ? 'successfully saved to' : 'failure when saving to') . ' ' . ($isFromSessDat ? 'session' : 'db'));

		return $this->resp;
	}

}
