<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: b10ef134afaa $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-10-12 10:52 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//we_modules_info::isActive(we_modules_info::SCHEDULER);

class we_schedule_scheduler{
	protected const SCHEDULE_FROM = 'publish'; //publish
	protected const SCHEDULE_TO = 'park'; //park
	protected const DELETE = 'delete';
	protected const DOCTYPE = 'doctype';
	protected const CATEGORY = 'category';
	protected const DIR = 'directory';
	protected const SEARCHABLE_ENABLED = 'search_enable';
	protected const SEARCHABLE_DISABLED = 'search_disable';
	protected const CALL = 'call';
	public const RERUN_ONCE = 'once'; //public due to updater
	public const RERUN_HOUR = 'hour'; //public due to updater
	public const RERUN_DAY = 'day'; //public due to updater
	public const RERUN_WEEK = 'week'; //public due to updater
	public const RERUN_MONTH = 'month'; //public due to updater
	public const RERUN_YEAR = 'year'; //public due to updater
	public const TRIGGER_PREDOC = 0;
	public const TRIGGER_POSTDOC = 1;
	public const TRIGGER_CRON = 2;
	protected const SUBMIT_DAYS = 'days';
	protected const SUBMIT_MONTHS = 'months';
	protected const SUBMIT_WDAYS = 'weekdays';
	protected const SUBMIT_TIME = 'time';
	protected const SUBMIT_RERUN = 'rerun';
	protected const SUBMIT_TASK = 'task';
	protected const SUBMIT_ACTIVE = 'active';
	protected const SUBMIT_DOCTYPE_DEFAULT = 'DoctypeDefault';
	protected const SUBMIT_DOCTYPE = 'DoctypeID';
	protected const SUBMIT_PARENT = 'ParentID';
	protected const SUBMIT_CATEGORY = 'CategoryIDs';
	const persistent_slots = [
		self::SUBMIT_DAYS => we_base_request::INTLISTA,
		self::SUBMIT_MONTHS => we_base_request::INTLISTA,
		self::SUBMIT_WDAYS => we_base_request::INTLISTA,
		//self::SUBMIT_TIME => we_base_request::DATE_TIME,
		self::SUBMIT_RERUN => we_base_request::STRING,
		self::SUBMIT_TASK => we_base_request::STRING,
		self::SUBMIT_ACTIVE => we_base_request::BOOL,
		self::SUBMIT_DOCTYPE_DEFAULT => we_base_request::BOOL,
		self::SUBMIT_DOCTYPE => we_base_request::INT,
		self::SUBMIT_PARENT => we_base_request::INT,
		self::SUBMIT_CATEGORY => we_base_request::INTLISTA,
	];

	/**
	 *
	 * @param int $id
	 * @param we_database_base $DB_WE
	 * @return void
	 */
	private static function processSchedule(int $id, we_database_base $DB_WE, string $ct, string $expire, string $table, we_schedule_entry $s): void{
		we_base_WE::$schedulerActive = true;
		try{
			$doc_save = we_base_WE::getWeDoc();
		} catch (we_exception_base $e){
			$doc_save = null;
		}
		$doc = we_base_ContentTypes::getObject($ct);
		if($doc instanceof we_document_webEdition){
			we_base_WE::setWeDoc($doc);
		}
		assert($doc instanceof we_document_base);
		$doc->InitByID($id, '', we_contents_base::LOAD_TEMP_DB);
		$callPublish = true;
		$changeTmpDoc = false;
		$_SESSION['weS']['versions']['fromScheduler'] = true;

		switch($s->task){
			case self::DELETE:
				we_base_delete::deleteEntry($id, $table);
				$callPublish = false;
				$changeTmpDoc = false;
				break; //exit foreach
			case self::SCHEDULE_FROM:
				$doc->Publish_Date = new DateTime();
				break;
			case self::SCHEDULE_TO:
				$doc->Publish_Date = null;
				break;
			case self::DOCTYPE:
				if($doc instanceof we_document_webEdition){
					$publSave = $doc->Publish_Date;
					if($doc->Publish_Date){
						$doc->we_unpublish();
					}
					$doc->DocType = $s->doctypeID;
					if($s->doctypeDefault){
						$doc->changeDoctype($s->doctypeID, true);
					}
					$changeTmpDoc = true;
					$doc->Publish_Date = $publSave;
				}
				break;
			case self::CATEGORY:
				$doc->Category = $s->categoryIDs;
				$changeTmpDoc = true;
				break;
			case self::DIR:
				$doc->setParentID((int) $s->parentID);
				$changeTmpDoc = true;
				break;
			case self::SEARCHABLE_ENABLED:
				$doc->IsSearchable = true;
				$changeTmpDoc = true;
				break;
			case self::SEARCHABLE_DISABLED:
				$doc->IsSearchable = false;
				$changeTmpDoc = true;
				break;
			case self::CALL:
				if(!$doc->Publish_Date){//don't call a document which is not published
					break;
				}
				$callPublish = false; //(count($s) > 1); //only if other operations pending
				//don't show any output
				ob_start();
				//we can't use include-tag since we don't have a document & many warnings will occur.
				$path = id2path($id, FILE_TABLE);
				if($path){
					include(WEBEDITION_PATH . '../' . $path);
				}
				ob_end_clean();
				break;
		}

		if($s->rerun != self::RERUN_ONCE){
			$nextWann = $s->getNextTimestamp();
			$DB_WE->query('UPDATE ' . SCHEDULE_TABLE . ' SET `expire`="' . $nextWann->format(we_database_base::DATE_TIME_FORMAT) . '" WHERE Active=1 AND DID=' . $id . ' AND documentTable="' . $table . '" AND rerun="' . $s->rerun . '" AND task="' . $s->task . '" AND `expire`="' . $expire . '"');
		} else {
			$DB_WE->query('UPDATE ' . SCHEDULE_TABLE . ' SET Active=0 WHERE Active=1 AND DID=' . $id . ' AND documentTable="' . $table . '" AND rerun="' . $s->rerun . '" AND task="' . $s->task . '" AND `expire`="' . $expire . '"');
		}

		if($changeTmpDoc){
			if(!$doc->we_save()){
				t_e('while scheduled save of document');
			}
		}

		if($callPublish){
			$pub = ($doc->Publish_Date ?
				$doc->we_publish() :
				$doc->we_unpublish());

			if(!$pub){
				t_e('Error while scheduled publish/unpublish of document', $doc->getErrMsg(), we_base_WE::$we_doc);
			}
		}

		we_base_WE::setWeDoc($doc_save);

		$_SESSION['weS']['versions']['fromScheduler'] = false;

		we_base_WE::$schedulerActive = false;
	}

	public static function trigger_schedule(int $limit = 10): void{
		//sth. to do???
		if(MAINTENANCE_MODE || !fB('SELECT 1 FROM ' . SCHEDULE_TABLE . ' WHERE `expire`<=NOW() AND lockedUntil<NOW() AND Active=1 LIMIT 1')){
			return;
		}

		$req = [empty($_REQUEST) ? [] : $_REQUEST, empty($_GET) ? [] : $_GET, empty($_POST) ? [] : $_POST];
		$_REQUEST = $_GET = $_POST = [];

		$DB_WE = new DB_WE();
		$DB_WE->addTable('del', [
			'ID' => 'int unsigned NOT NULL',
			'documentTable' => 'enum("tblFile","tblObjectFiles") NOT NULL'
			], ['PRIMARY KEY (ID,documentTable)'], 'MEMORY', true);
		$DB_WE->query('INSERT INTO del (ID,documentTable) SELECT s.DID,s.documentTable FROM ' . SCHEDULE_TABLE . ' s LEFT JOIN ' . FILE_TABLE . ' f ON f.ID=s.DID ' .
			(we_modules_info::isActive(we_modules_info::OBJECT) ? ' LEFT JOIN ' . OBJECT_FILES_TABLE . ' `of` ON `of`.ID=s.DID' : '') . ' WHERE (f.ID IS NULL AND s.documentTable="tblFile")' . (we_modules_info::isActive(we_modules_info::OBJECT) ? ' OR (`of`.ID IS NULL AND s.documentTable="tblObjectFiles")' : '') . ' GROUP BY s.DID,s.documentTable');
		$DB_WE->query('DELETE FROM ' . SCHEDULE_TABLE . ' WHERE (DID,documentTable) IN (SELECT ID,documentTable FROM del )');
		$DB_WE->delTable('del', true);

		$hasLock = $DB_WE->hasLock();
		//allow at max 10 scheduled activities per call, in case no cron is used.
		$maxSched = defined('SCHEDULED_BY_CRON') ? -1 : $limit + 1;
//make sure documents don't know they are inside WE
		$lastWEState = [];
		if(we_base_WE::$mainDoc['editmode'] || we_base_WE::$editmode){
			$lastWEState = [
				'WE_MAIN_EDITMODE' => (we_base_WE::$mainDoc['editmode']),
				'editmode' => we_base_WE::$editmode,
				'template' => we_base_WE::$template,
			];
			we_base_WE::$mainDoc['editmode'] = we_base_WE::$editmode = false;
		}

		while((!$hasLock || $DB_WE->lock([SCHEDULE_TABLE, SCHEDULE_TABLE . ' AS s', FILE_TABLE . ' f', ERROR_LOG_TABLE])) && ( --$maxSched != 0) && ($rec = getHash('SELECT
	s.Active,
	s.DID,
	s.documentTable,
	s.expire,
	s.task,
	s.ParentID,
	s.DoctypeID,
	s.DoctypeDefault,
	s.CategoryIDs,
	s.rerun,
	s.`time`,
	lpad(IFNULL(bin(s.`months`),0),12,"0") AS `months`,
	lpad(IFNULL(bin(s.`days`),0),31,"0") AS `days`,
	lpad(IFNULL(bin(s.`weekdays`),0),7,"0") AS `weekdays`,
	f.ContentType
FROM ' .
		SCHEDULE_TABLE . ' s LEFT JOIN ' .
		FILE_TABLE . ' f ON (f.ID=s.DID AND s.documentTable="tblFile")
WHERE
	s.`expire`<=NOW() AND
	s.lockedUntil<NOW() AND
	s.Active=1
ORDER BY s.`expire` LIMIT 1', $DB_WE))){
			$DB_WE->query('UPDATE ' . SCHEDULE_TABLE . ' s SET lockedUntil=NOW()+INTERVAL 1 minute WHERE DID=' . $rec['DID'] . ' AND Active=1 AND documentTable="' . $rec['documentTable'] . '" AND rerun="' . $rec['rerun'] . '" AND task="' . $rec['task'] . '" AND `expire`="' . $rec['expire'] . '"');
			if($hasLock){
				$DB_WE->unlock();
			}
			$s = we_schedule_entry::fromHash($rec);

			//self::getPrevTimestamp($s);
			we_base_WE::resetTemplate();
			we_base_WE::resetMainDoc();
			self::processSchedule($rec['DID'], $DB_WE, $rec['ContentType'] ?: we_base_ContentTypes::OBJECT_FILE, $rec['expire'], $rec['documentTable'] === 'tblObjectFiles' ? OBJECT_FILES_TABLE : FILE_TABLE, $s);
		}
		//cleanup old single shots
		$DB_WE->query('DELETE FROM ' . SCHEDULE_TABLE . ' WHERE Active=0 AND rerun="' . self::RERUN_ONCE . '" AND `expire`<(CURDATE()-INTERVAL 1 WEEK)');
		//make sure DB is unlocked!
		$DB_WE->unlock();
//reset state
		[$_REQUEST, $_GET, $_POST] = $req;
		if($lastWEState){
			we_base_WE::$mainDoc['editmode'] = $lastWEState['WE_MAIN_EDITMODE'];
			$GLOBALS['we_editmode'] = we_base_WE::$editmode = $lastWEState['editmode'];
			we_base_WE::$template = $lastWEState['template'];
		}
	}

	static function isSaveInSchedulerActive(we_contents_root $object): bool{
		if(!($object instanceof we_document_base) || empty($object->schedArr)){
			return false;
		}
		foreach($object->schedArr as $s){
			if($s->isSchedulerActive()){
				return true;
			}
		}
		return false;
	}

	/**
	 *
	 * @param int $id
	 * @param string $table
	 * @param we_schedule_entry[] $schedArr
	 * @param we_database_base $db
	 * @return ?DateTime
	 */
	public static function publInScheduleTable(int $id, string $table, array $schedArr, we_database_base $db): ?DateTime{
		//FIXME: don't delete and insert if entry is not changed....
		$db->query('DELETE FROM ' . SCHEDULE_TABLE . ' WHERE DID=' . $id . ' AND documentTable="' . stripTblPrefix($table) . '"');
		$makeSched = [];
		foreach($schedArr as $s){
			try{
				$dat = $s->getSaveData($id, $table);
			} catch (we_exception_base $e){//no valid next time found
				continue;
			}
			if($s->task === self::SCHEDULE_FROM){
				$makeSched[] = $dat['expire'];
			}
			if(!$db->query('INSERT INTO ' . SCHEDULE_TABLE . ' SET ' . we_database_base::arraySetter($dat))){
				throw new we_exception_database('error inserting schedule');
			}
		}
		if($makeSched){
			return new DateTime(min($makeSched));
		}
		return null;
	}

	/**
	 *
	 * @param array $req
	 * @return we_schedule_entry[]
	 */
	public static function getScheduleFromRequest(array $req): array{
		$ret = [];
		foreach(array_keys($req) as $pos){
			$ret[$pos] = we_schedule_entry::fromRequest((int) $pos);
		}
		return $ret;
	}

	/**
	 * @param int $id
	 * @param string $table
	 * @return we_schedule_entry[]
	 */
	public static function getScheduleTable(int $id, string $table): array{
		$ret = [];
		$recs = DB_WE::$inst->getAllq('SELECT
	s.Active,
	s.rerun,
	s.task,
	s.ParentID,
	s.DoctypeID,
	s.DoctypeDefault,
	s.CategoryIDs,
	s.time,
	lpad(IFNULL(bin(s.`months`),0),12,"0") AS `months`,
	lpad(IFNULL(bin(s.`days`),0),31,"0") AS `days`,
	lpad(IFNULL(bin(s.`weekdays`),0),7,"0") AS `weekdays`
FROM ' .
			SCHEDULE_TABLE . ' s WHERE
	s.DID=' . $id . ' AND
	s.documentTable="' . stripTblPrefix($table) . '"');
		'@phan-var array<int,array{Active:int,rerun:string,task:string,ParentID:int,DoctypeID:int,DoctypeDefault:int,CategoryIDs:string,time:string,months:string,days:string,weekdays:string}> $recs';

		foreach($recs as $rec){
			$ret[] = we_schedule_entry::fromHash($rec);
		}
		return $ret;
	}

}
