<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: ead093ffe1e6 $
 * $Author: Lukas Imhof $
 * $Date: 2020-10-01 21:52 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_search_frames extends we_modules_frame{
	const TAB_DOCUMENTS = 1;
	const TAB_TEMPLATES = 2;
	const TAB_MEDIA = 5;
	const TAB_ADVANCED = 3;
	const TAB_PROPERTIES = 4;

	/** @var we_search_view */
	protected $View;

	/** @var we_search_tree */
	protected $Tree;

	public function __construct(){
		$this->module = we_modules_info::SEARCH;
		parent::__construct();
		$this->Tree = new we_search_tree($this->jsCmd);

		$model = empty($_SESSION['weS']['module_sessions']['search']) ? new we_search_model() : $_SESSION['weS']['module_sessions']['search'];
		$this->View = new we_search_view($model);
	}

	protected function getHTMLCmd(): string{
		if(($pid = we_base_request::int('pid', -1)) < 0){
			return $this->getHTMLDocument(we_html_element::body(), $this->jsCmd->getCmds());
		}

		$offset = we_base_request::int('offset', 0);
		$attr = [];

		if(!empty($_SESSION['weS']['search']['modelidForTree'])){
			$attr['onload'] = "top.content.treeData.selectNode(top.content.treeData.table,'" . ($_SESSION['weS']['search']['modelidForTree']) . "');";
			unset($_SESSION['weS']['search']['modelidForTree']);
		}
		$this->jsCmd->addCmd('loadTree', ['clear' => !$pid, 'items' => we_search_tree::getItems($pid, $offset, $this->Tree->default_segment)]);

		return $this->getHTMLDocument(
				we_html_element::body($attr, we_html_element::form(['name' => we_base_constants::WE_FORM
						], we_html_element::hiddens(['pnt' => 'cmd',
							'cmd' => 'no_cmd'])
		)));
	}

	protected function getHTMLEditorHeader(): string{
		$we_tabs = new we_gui_tabs();

		if(!$this->View->Model->IsFolder){
			$displayEntry = 'inline';
			$displayFolder = 'none';
		} else {
			//folders and entries have different tabs to display
			$displayEntry = 'none';
			$displayFolder = 'inline';
		}

		//tabs for entries
		if(we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE])){
			$we_tabs->addTab(g_l('searchtool', '[documents]'), 'fa-file-o', false, (string) (self::TAB_DOCUMENTS), ['id' => 'tab_1', 'style' => 'display:' . $displayEntry]);
		}
		if(we_editor_functions::isNormalMode() && we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_CAN_SEE])){
			$we_tabs->addTab(g_l('searchtool', '[templates]'), 'fa-file-code-o', false, (string) (self::TAB_TEMPLATES), ['id' => 'tab_2',
				'style' => 'display:' . $displayEntry]);
		}
		if(we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE])){// FIXME: add some media related perm
			$we_tabs->addTab(g_l('searchtool', '[media]'), 'fa-image', false, (string) (self::TAB_MEDIA), ['id' => 'tab_5', 'style' => 'display:' . $displayEntry]);
		}
		$we_tabs->addTab(g_l('searchtool', '[advSearch]'), 'fa-search-plus', false, (string) (self::TAB_ADVANCED), ['id' => 'tab_3',
			'style' => 'display:' . $displayEntry]);

		//tabs for folders
		$we_tabs->addTab(g_l('searchtool', '[properties]'), '', false, (string) (self::TAB_PROPERTIES), ['id' => 'tab_4', 'style' => 'display:' . $displayFolder]);

		$tabNr = $this->getTab();
		$this->jsCmd->addCmd('setTab', $tabNr);

		$tabsHead = we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
			we_html_element::jsScript(JS_DIR . 'initTabs.js') .
			we_html_element::jsScript(WE_JS_MODULES_DIR . 'search/search_view2.js');

		$Text = we_search_model::getLangText($this->View->Model->Path, $this->View->Model->Text);
		$body = we_html_element::body(
				['id' => 'eHeaderBody',
					'onload' => "weTabs.setFrameSize();document.getElementById('tab_'+top.content.activ_tab).className='tabActive';",
					'onresize' => 'weTabs.setFrameSize()'
				], '<div id="main"><div id="headrow">&nbsp;' .
				we_html_element::b(g_l('searchtool', ($this->View->Model->IsFolder ? '[topDir]' : '[topSuche]')) . ':&nbsp;' . $Text . '<div id="mark"><i class="fa fa-asterisk modified"></i></div>') .
				'</div>' .
				$we_tabs->getHTML() .
				'</div>');

		return $this->getHTMLDocument($body, $tabsHead);
	}

	protected function getHTMLEditorBody(): string{
		$body = we_html_element::body(
				['class' => 'weEditorBody',
					'onkeypress' => "if(event.keyCode==='Enter'){weSearch.search(true);}",
					'onload' => "loaded=1;setTimeout(weSearch.init,200);",
					'onresize' => "weSearch.sizeScrollContent();"
				], we_html_element::form(['name' => we_base_constants::WE_FORM,
					'onsubmit' => 'return false'
					], $this->getHTMLProperties() .
					we_html_element::hiddens(['predefined' => $this->View->Model->predefined,
						'savedSearchName' => $this->View->Model->Text]
				))
		);

		$whichSearch = we_search_view::SEARCH_DOCS;
		$tabNr = $this->getTab();

		switch($tabNr){
			case self::TAB_DOCUMENTS:
				$whichSearch = we_search_view::SEARCH_DOCS;
				break;
			case self::TAB_TEMPLATES:
				$whichSearch = we_search_view::SEARCH_TMPL;
				break;
			case self::TAB_ADVANCED:
				$whichSearch = we_search_view::SEARCH_ADV;
				break;
			case self::TAB_MEDIA:
				$whichSearch = we_search_view::SEARCH_MEDIA;
				break;
		}

		return $this->getHTMLDocument($body, $this->View->getJSProperty() . $this->View->getSearchJS($whichSearch));
	}

	private function getTab(): int{
		if(($cmdid = we_base_request::int('cmdid', -1))){
			$_REQUEST['searchstartAdvSearch'] = 0;
		}
		if(($tab = we_base_request::int('tab'))){
			return $tab;
		}
		if($cmdid !== -1){
			return $this->View->Model->activTab;
		}

		// FIXME: set whichserach as new model property once in the beginning and forget about such rubish like that...
		return we_base_request::int('tabnr') ?: ($this->View->Model->getPropertyInt('activTab') ?: self::TAB_DOCUMENTS);
	}

	private function getHTMLProperties(): string{
		$tabNr = $this->getTab();

		return $this->View->getCommonHiddens([
				'cmd' => '',
				'pnt' => 'edbody',
				'tabnr' => $tabNr,
			]) .
			we_html_element::hidden('newone', ($this->View->Model->ID == 0 ? '1' : '0')) .
			we_html_element::div(['id' => 'tab1', 'style' => ($tabNr == self::TAB_DOCUMENTS ? 'display: block;' : 'display: none')], $tabNr == self::TAB_DOCUMENTS ? $this->getHTMLSearchtool($this->getHTMLTabDocuments()) : '') .
			we_html_element::div(['id' => 'tab2', 'style' => ($tabNr == self::TAB_TEMPLATES ? 'display: block;' : 'display: none')], $tabNr == self::TAB_TEMPLATES ? $this->getHTMLSearchtool($this->getHTMLTabTemplates()) : '') .
			we_html_element::div(['id' => 'tab5', 'style' => ($tabNr == self::TAB_MEDIA ? 'display: block;' : 'display: none')], $tabNr == self::TAB_MEDIA ? $this->getHTMLSearchtool($this->getHTMLTabMedia()) : '') .
			we_html_element::div(['id' => 'tab3', 'style' => ($tabNr == self::TAB_ADVANCED ? 'display: block;' : 'display: none')], $tabNr == self::TAB_ADVANCED ? $this->getHTMLSearchtool($this->getHTMLTabAdvanced()) : '') .
			we_html_element::div(['id' => 'tab4', 'style' => ($tabNr == self::TAB_PROPERTIES ? 'display: block;' : 'display: none')], $this->getHTMLSearchtool($this->getHTMLGeneral()));
	}

	private function getHTMLGeneral(): array{
		$disabled = true;
		$this->View->Model->Text = we_search_model::getLangText($this->View->Model->Path, $this->View->Model->Text);

		return [
			new we_html_multiIconBoxContent(),
				(new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('searchtool', '[general]'), we_html_tools::formElementTable(g_l('searchtool', '[dir]'), we_html_tools::textInput('Text', $this->View->Model->Text, 0, [
								'style' => "width: 520px"], '', '0', false, $disabled))))
					->space(we_html_multiIconBox::SPACE_MED)
					->noline())
		];
	}

	private function getHTMLTabDocuments(): array{
		$whichSearch = we_search_view::SEARCH_DOCS;
		$resultContainer = $this->View->makeResultContainer($whichSearch);
		$foundItems = $_SESSION['weS']['search']['foundItems' . $whichSearch] ?? 0;

		return [(new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('searchtool', '[text]'), '<div>' . $this->View->getSearchDialog($whichSearch) . '</div>', we_html_multiIconBox::PROP_DOC))
					->noline())
				->add((new we_html_multiIconBoxContentLine(g_l('searchtool', '[suchenIn]'), '<div>' . $this->View->getDirSelector($whichSearch) . '</div>', we_html_multiIconBox::PROP_PATH))
					->noline())
				->add((new we_html_multiIconBoxContentLine(g_l('searchtool', '[optionen]'), '<div>' . $this->View->getSearchDialogOptions($whichSearch) . '</div>', we_html_multiIconBox::PROP_ATTRIB)
				)),
				(new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine('', '<div>' . we_html_element::div(['id' => 'mouseOverDivs_' . we_search_view::SEARCH_DOCS]) . '
		<div id="parametersTop_' . $whichSearch . '">' . $this->View->getSearchParameterTop($foundItems, $whichSearch) . '</div>' . $resultContainer . '<div id="parametersBottom_' . $whichSearch . '">' . $this->View->getSearchParameterBottom($foundItems, $whichSearch) . '</div>
		</div>', we_html_multiIconBox::PROP_RESULT))
		)];
	}

	private function getHTMLTabTemplates(): array{
		$whichSearch = we_search_view::SEARCH_TMPL;
		$resultContainer = $this->View->makeResultContainer($whichSearch);
		$foundItems = $_SESSION['weS']['search']['foundItemsTmplSearch'] ?? 0;

		return [
				(new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('searchtool', '[text]'), '<div>' . $this->View->getSearchDialog($whichSearch) . '</div>', we_html_multiIconBox::PROP_DOC))
					->noline())
				->add((new we_html_multiIconBoxContentLine(g_l('searchtool', '[suchenIn]'), '<div>' . $this->View->getDirSelector($whichSearch) . '</div>', we_html_multiIconBox::PROP_PATH))
					->noline())
				->add((new we_html_multiIconBoxContentLine(g_l('searchtool', '[optionen]'), '<div>' . $this->View->getSearchDialogOptions($whichSearch) . '</div>', we_html_multiIconBox::PROP_ATTRIB)
				)),
				(new we_html_multiIconBoxContent())
				->add(new we_html_multiIconBoxContentLine('', '<div>' . we_html_element::div(['id' => 'mouseOverDivs_' . we_search_view::SEARCH_TMPL]) . '
		<div id="parametersTop_' . $whichSearch . '">' . $this->View->getSearchParameterTop($foundItems, $whichSearch) . '</div>' . $resultContainer . '<div id="parametersBottom_' . $whichSearch . '">' . $this->View->getSearchParameterBottom($foundItems, $whichSearch) . '</div>
		</div>', we_html_multiIconBox::PROP_RESULT
				))
		];
	}

	private function getHTMLTabMedia(): array{
		$whichSearch = we_search_view::SEARCH_MEDIA;
		$resultContainer = $this->View->makeResultContainer($whichSearch);
		$foundItems = $_SESSION['weS']['search']['foundItems' . $whichSearch] ?? 0;

		return [
				(new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('searchtool', '[text]'), '<div>' . $this->View->getSearchDialog($whichSearch) . '</div>', we_html_multiIconBox::PROP_DOC))
					->noline())
				->add((new we_html_multiIconBoxContentLine(g_l('searchtool', '[suchenIn]'), '<div>' . $this->View->getDirSelector($whichSearch) . '</div>', we_html_multiIconBox::PROP_PATH))
					->noline())
				->add((new we_html_multiIconBoxContentLine(g_l('searchtool', '[optionen]'), '<div>' . $this->View->getSearchDialogOptions($whichSearch) . '</div>', we_html_multiIconBox::PROP_ATTRIB))
					->noline())
				->add((new we_html_multiIconBoxContentLine(g_l('searchtool', '[anzeigen]'), '<div>' . $this->View->getSearchDialogMediaType($whichSearch) . '</div>', we_html_multiIconBox::PROP_DISPLAY))
					->noline())
				->add(new we_html_multiIconBoxContentLine(g_l('searchtool', '[filter]'), '<div>' . $this->View->getSearchDialogFilter($whichSearch) . '</div>', we_html_multiIconBox::PROP_FILTER)),
			(new we_html_multiIconBoxContent())
				->add(new we_html_multiIconBoxContentLine('', '<div>' . we_html_element::div(['id' => 'mouseOverDivs_' . we_search_view::SEARCH_MEDIA]) . '
		<div id=\'parametersTop_' . $whichSearch . '\'>' . $this->View->getSearchParameterTop($foundItems, $whichSearch) . '</div>' . $resultContainer . '<div id=\'parametersBottom_' . $whichSearch . '\'>' . $this->View->getSearchParameterBottom($foundItems, $whichSearch) . '</div>
	</div>', we_html_multiIconBox::PROP_RESULT))
		];
	}

	private function getHTMLTabAdvanced(): array{
		$whichSearch = we_search_view::SEARCH_ADV;
		$resultContainer = $this->View->makeResultContainer($whichSearch);
		$foundItems = $_SESSION['weS']['search']['foundItems' . we_search_view::SEARCH_ADV] ?? 0;

		return [
				(new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(''/* g_l('searchtool', '[text]'), */, '<div>' . $this->View->getSearchDialogOptionalFields(we_search_view::SEARCH_ADV) . '</div>', we_html_multiIconBox::PROP_DOC))
					->noline())
				->add((new we_html_multiIconBoxContentLine(g_l('searchtool', '[anzeigen]'), '<div>' . $this->View->getSearchDialogCheckboxesAdvSearch() . '</div>', we_html_multiIconBox::PROP_DISPLAY))),
			(new we_html_multiIconBoxContent())
				->add(new we_html_multiIconBoxContentLine('', '<div>' . we_html_element::div(['id' => 'mouseOverDivs_' . we_search_view::SEARCH_ADV]) . '
      <div id=\'parametersTop_' . we_search_view::SEARCH_ADV . '\'>' . $this->View->getSearchParameterTop($foundItems, we_search_view::SEARCH_ADV) . '</div>' .
						$resultContainer . '<div id=\'parametersBottom_' . we_search_view::SEARCH_ADV . '\'>' . $this->View->getSearchParameterBottom($foundItems, we_search_view::SEARCH_ADV) . '</div>
			</div>', we_html_multiIconBox::PROP_RESULT))
		];
	}

	private function getHTMLSearchtool(array $content): string{
		return
			(new we_html_multiIconBox($content[0], 'contentParameters'))->marginLeft()->fold(1, true, g_l('searchtool', '[suchen]'), g_l('searchtool', '[suchen]'), 'weSearch.sizeScrollContent();')->get() .
			(new we_html_multiIconBox($content[1], 'contentResult'))->marginLeft()->get();
	}

	public function getHTML(string $what = '', int $mode = 0, int $step = 0): string{
		switch($what){
			case 'frameset':
				return $this->getHTMLFrameset('', ($tab = we_base_request::int('tab', -1)) > -1 ? '&tab=' . $tab : '');
			case 'editor':
				return $this->getHTMLEditor(($tab = we_base_request::int('tab', -1)) > -1 ? '&tab=' . $tab : '');
			case self::FOOTER:
				return $this->getHTMLEditorFooter([
						'main' => [we_html_button::FOOTER_SAVE => [[], ['weSearch_save']]]
				]);
			default:
				return parent::getHTML($what, $mode, $step);
		}
	}

	protected function getHTMLFrameset(string $extraHead = '', string $extraUrlParams = ''): string{
		return parent::getHTMLFrameset($this->Tree->getJSTreeCode() . $extraHead, $extraUrlParams);
	}

}
