<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 3659f7583a8d $
 * $Author: Lukas Imhof $
 * $Date: 2020-08-06 12:49 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_sendMail extends we_weTag_tag{
	/** @var bool */
	public static $active = false;

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['recipient' => false, 'from' => false], __CLASS__);
		self::$doc = we_base_WE::getWeDoc();

		if(self::$doc->InWebEdition){
			return '';
		}

		$id = self::getAttribute('id', $attribs, we_base_request::int('ID', 0), we_base_request::INT);
		if(!$id){
			return '';
		}
		$attachPath = self::getAttribute('attachments', $attribs, [], we_base_request::STRING_LIST);
		$attachID = id2pathA(self::getAttribute('attachmentIDs', $attribs, [], we_base_request::INTLISTA));

		$attach = array_filter(array_merge($attachPath, $attachID));

		$from = self::getAttribute('from', $attribs, '', we_base_request::EMAIL);
		$reply = self::getAttribute('reply', $attribs, '', we_base_request::EMAIL);
		$recipient = self::getAttribute('recipient', $attribs, '', we_base_request::STRING); //FIXME:email_list
		$recipientCC = self::getAttribute('recipientcc', $attribs, self::getAttribute('recipientCC', $attribs, '', we_base_request::STRING), we_base_request::STRING, true); //FIXME:email_list
		$recipientBCC = self::getAttribute('recipientbcc', $attribs, self::getAttribute('recipientBCC', $attribs, '', we_base_request::STRING), we_base_request::STRING, true); //FIXME:email_list

		$mimetype = self::getAttribute('mimetype', $attribs, '', we_base_request::STRING);
		$subject = self::getAttribute('subject', $attribs, '', we_base_request::STRING);
		$includeimages = self::getAttribute('includeimages', $attribs, false, we_base_request::BOOL);
		$useBaseHref = self::getAttribute('usebasehref', $attribs, true, we_base_request::BOOL);
		$useFormmailLog = self::getAttribute('useformmaillog', $attribs, false, we_base_request::BOOL);
		$useFormmailBlock = self::getAttribute('useformmailblock', $attribs, false, we_base_request::BOOL);
		if($useFormmailBlock){
			$useFormmailLog = true;
		}
		$blocked = false;

		$we_recipient = self::getMails(explode(',', $recipient));
		$we_recipientCC = self::getMails(explode(',', $recipientCC));
		$we_recipientBCC = self::getMails(explode(',', $recipientBCC));

		if($useFormmailLog){
			// insert into log
			DB_WE::$inst->query('INSERT INTO ' . FORMMAIL_LOG_TABLE . ' SET IP="' . DB_WE::$inst->escape($_SERVER['REMOTE_ADDR']) . '"');
			if(FORMMAIL_EMPTYLOG > -1){
				DB_WE::$inst->query('DELETE FROM ' . FORMMAIL_LOG_TABLE . ' WHERE unixTime<(NOW()-INTERVAL ' . intval(FORMMAIL_EMPTYLOG) . ' SECOND)');
			}

			if($useFormmailBlock){
				$trials = FORMMAIL_TRIALS;
				// first delete all entries from blocktable which are older then now - blocktime
				DB_WE::$inst->query('DELETE FROM ' . FORMMAIL_BLOCK_TABLE . ' WHERE blockedUntil!=-1 AND blockedUntil<UNIX_TIMESTAMP()');

				// check if ip is allready blocked
				if(fB('SELECT 1 FROM ' . FORMMAIL_BLOCK_TABLE . ' WHERE ip="' . DB_WE::$inst->escape($_SERVER['REMOTE_ADDR']) . '" LIMIT 1')){
					$blocked = true;
				} else {
					// ip is not blocked, so see if we need to block it
					if(fI('SELECT COUNT(1) FROM ' . FORMMAIL_LOG_TABLE . ' WHERE unixTime>(NOW()- INTERVAL ' . intval(FORMMAIL_SPAN) . ' SECOND) AND ip="' . DB_WE::$inst->escape($_SERVER['REMOTE_ADDR']) . '"') > $trials){
						$blocked = true;
						// insert in block table
						$blockedUntil = (FORMMAIL_BLOCKTIME == -1) ? -1 : '(UNIX_TIMESTAMP()+' . FORMMAIL_BLOCKTIME . ')';
						DB_WE::$inst->query('REPLACE INTO ' . FORMMAIL_BLOCK_TABLE . " (ip, blockedUntil) VALUES('" . DB_WE::$inst->escape($_SERVER['REMOTE_ADDR']) . "', " . $blockedUntil . ")");
					}
				}
			}
		}
		if($blocked){
			$headline = 'Fehler / Error';
			$content = g_l('global', '[formmailerror]') . self::htmlTag('br') . '&#8226; ' . 'Email dispatch blocked / Email Versand blockiert!';

			echo we_html_tools::getHtmlTop('', '', self::htmlTag('body', ['class' => 'weEditorBody'], we_html_tools::dialogLayout(self::htmlTag('div', ['class' => 'defaultfont lowContrast'], $content), $headline)));

			exit;
		}
		if(!isset($_SESSION)){
			we_base_WE::$session = new we_base_sessionHandler();
		}
		self::$active = true;
		$codes = we_base_file::isWeFile($id, FILE_TABLE) ? we_getDocumentByID($id) : '';
		self::$active = false;
		if(!$codes){
			t_e('Document to send via we:sendMail is empty ID: ' . $id);
		}
		$phpmail = new we_mail_mail($we_recipient, $subject, we_mail_address::parse($from), we_mail_address::parse($reply), $includeimages);
		if(isset($includeimages)){
			$phpmail->setIsEmbedImages($includeimages);
		}
		if(($we_recipientCC)){
			$phpmail->setCC($we_recipientCC);
		}
		if(($we_recipientBCC)){
			$phpmail->setBCC($we_recipientBCC);
		}
		if(isset($useBaseHref)){
			$phpmail->setIsUseBaseHref($useBaseHref);
		}
		if($mimetype != we_mail_mime::TYPE_HTML){
			$phpmail->setTextPartOutOfHTML($codes);
		} else {
			$phpmail->addHTMLPart($codes);
		}
		// sendMailAttach: Anhaenge hinzufuegen
		foreach($attach as $filename){
			$phpmail->doaddAttachment(WE_DOCUMENT_ROOT . $filename);
		}

		$phpmail->buildMessage();
		$phpmail->send();
		return '';
	}

	protected static function getMails(array $to): array{
		$we_recipient = [];
		foreach($to as $mail){
			assert(is_string($mail));
			if(strpos($mail, '@') === false){
				$mailA = we_base_request::_(we_base_request::EMAIL, $mail, we_base_request::NOT_VALID_S);
				if((
					(!empty($_SESSION['webuser']['registered'])) ||
					(we_tag_customerResetPassword::$status === we_customer_customer::PWD_ALL_OK)) &&
					isset($_SESSION['webuser'][$mail]) && strpos($_SESSION['webuser'][$mail], '@') !== false){ //wenn man registireten Usern was senden moechte
					if(we_check_email($_SESSION['webuser'][$mail])){
						$we_recipient[] = we_mail_address::parse($_SESSION['webuser'][$mail]);
					}
				} else if($mailA !== we_base_request::NOT_VALID_S){ //email to friend test
					if(we_check_email($mailA)){
						$we_recipient[] = we_mail_address::parse($mailA);
					}
				}
			} elseif(we_check_email($mail)){
				$we_recipient[] = we_mail_address::parse($mail);
			}
		}

		return $we_recipient;
	}

}
