<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: c564388e074d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-17 13:01 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_shipping extends we_weTag_tag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['sum' => false], __CLASS__);

		$sumName = self::getAttribute('sum', $attribs, '', we_base_request::STRING);

		// shipping depends on total value of basket
		$orderVal = we_tag_sum::getSum($sumName);
		if($orderVal === null){
			return '0';
		}

		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);

		$weShippingControl = we_shop_shippingControl::getShippingControl();

		// check if user is registered
		$customer = (we_weTag_util::tag('ifRegisteredUser') ? $_SESSION['webuser'] : []);

		$shippingCost = $weShippingControl->getShippingCostByOrderValue($orderVal, $customer);

		// get calculated value if needed
		// if user must NOT pay vat always return net prices
		$mustPayVat = we_weTag_util::tag('ifShopPayVat'); // alayways return net prices

		switch($type){
			case 'net':
				//no difference if $mustPayVat or not
				if(!$weShippingControl->isNet){
					// y = x * (100/116)
					$shippingCost *= (100 / ((1 + ($weShippingControl->vatRate / 100)) * 100) );
				}
				break;
			case 'gros':
				if($weShippingControl->isNet){
					// y = x * (1.16)
					$shippingCost = ($mustPayVat ?
						$shippingCost * (1 + ($weShippingControl->vatRate / 100)) :
						//return net prices
						$shippingCost * (100 / ((1 + ($weShippingControl->vatRate / 100)) * 100) )
						);
				}
				break;
			case 'vat':
				$shippingCost = ($mustPayVat ?
					($weShippingControl->isNet ?
					// y = x * 0.16
					$shippingCost * ($weShippingControl->vatRate / 100) :
					// y = x /116 * 16
					$shippingCost / ( ((1 + ($weShippingControl->vatRate / 100)) * 100) ) * $weShippingControl->vatRate) :
					0);
				break;
			default:
		}

		return we_base_util::formatNumber($shippingCost, self::getAttribute('num_format', $attribs, '', we_base_request::STRING));
	}

}
