<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 24bf12724cb1 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-09-03 21:17 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_weTag_selectorAttribute extends we_weTag_attribute{
	const FOLDER = 'folder';

	/** @var string	 */
	protected $Table;

	/** @var string	 */
	protected $Selectable;

	/** @var bool */
	private $useName;

	/**
	 * @param string $name
	 * @param string $table
	 * @param string $selectable
	 * @param boolean $required
	 */
	function __construct(string $name, string $table, string $selectable, bool $required = false, string $module = '', bool $useName = false, string $description = '', bool $deprecated = false){
		parent::__construct($name, $required, $module, $description, $deprecated);
		$this->Table = $table;
		$this->Selectable = $selectable;
		$this->useName = $useName;
	}

	/**
	 * @return string
	 */
	function getCodeForTagWizard(): string{
		switch($this->Table){
			case CATEGORY_TABLE:
				$weCmd = 'we_selector_document';
				$this->Selectable = '';
				break;
			case NAVIGATION_TABLE:
				$weCmd = 'we_selector_file';
				break;
			default:
				$weCmd = ($this->Selectable == self::FOLDER ? 'we_selector_directory' : 'we_selector_document');
		}

		$input = we_html_element::input([
				'name' => $this->Name,
				'value' => $this->Value,
				'id' => $this->getIdName(),
				'class' => 'wetextinput'
		]);
		$button = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('" . $weCmd . "', {id:document.getElementById('" . $this->getIdName() . "').value, table:'" . $this->Table . "', targetID:'" . ($this->useName ? '' : $this->getIdName()) . "', targetName:'" . ($this->useName ? $this->getIdName() : '') . "', filter:'" . $this->Selectable . "'})");

		return '<table class="attribute"><tr>
						<td class="attributeName">' . $this->getLabelCodeForTagWizard() . '</td>
						<td class="attributeField">' . $input . '</td>
						<td class="attributeButton">' . $button . '</td>
					</tr></table>';
	}

}
