<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 0627c1498cde $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-02 12:07 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_widget_usr extends we_widget_base{

	public function __construct(string $curID = '', array $aProps = []){

	}

	/**
	 * @return array{string,array{string,string}}
	 */
	public function getInsertDiv(int $iCurrId, array $aProps, we_base_jsCmd $jsCmd): array{
		if(!we_base_permission::hasPerm([we_base_perm::WIDGET_SHOW_USR])){
			throw new we_exception_permission('unallowed');
		}
		[$num, $usr] = we_users_online::getUsers();
		$oTblCont = new we_html_table(
			['id' => 'm_' . $iCurrId . '_inline',
			'style' => 'width:100%'
			], 1, 1);
		$oTblCont->setCol(0, 0, '<div id="users_online">' . $usr . '</div>');
		$aLang = [g_l('cockpit', '[users_online]'), ' (<span id="num_users">' . $num . '</span>)'];

		$oTblDiv = $oTblCont->getHtml();
		return [$oTblDiv, $aLang];
	}

	/**
	 * @return array<string,mixed>
	 */
	public static function getDefaultConfig(): array{
		return (we_base_permission::hasPerm([we_base_perm::WIDGET_SHOW_USR]) ? [
			'width' => self::WIDTH_SMALL,
			'expanded' => 0,
			'height' => 210,
			'res' => 0,
			'cls' => 'lightCyan',
			'csv' => '',
			'dlgHeight' => 140,
			'isResizable' => true
			] : []);
	}

	public static function showDialog(): void{
		[$jsFile, $oSelCls] = self::getDialogPrefs();

		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine('', $oSelCls->getHTML()));

		$save_button = we_html_button::create_button(we_html_button::SAVE, "javascript:save();");
		$preview_button = we_html_button::create_button(we_html_button::PREVIEW, "javascript:preview();");
		$cancel_button = we_html_button::create_button(we_html_button::CLOSE, "javascript:exit_close();");
		$buttons = we_html_button::position_yes_no_cancel($save_button, $preview_button, $cancel_button);

		$sTblWidget = (new we_html_multiIconBox($parts, 'usrProps'))
			->marginLeft()
			->buttons($buttons)
			->headline(g_l('cockpit', '[users_online]'))
			->get();

		echo we_html_tools::getHtmlTop(g_l('cockpit', '[users_online]'), $jsFile .
			we_html_element::jsScript(JS_DIR . 'widgets/usr.js'), we_html_element::body(
				['class' => 'weDialogBody', 'onload' => "init();"
				], we_html_element::form([], $sTblWidget)));
	}

	public function showPreview(): void{

	}

}
