/* global container, drawTree, WE, top, Node*/

/**
 * webEdition SDK
 *
 * webEdition CMS
 * $Author: Lukas Imhof $
 *
 * This source is part of the webEdition SDK. The webEdition SDK is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU Lesser General Public License can be found at
 * http://www.gnu.org/licenses/lgpl-3.0.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionSDK/License.txt
 *
 *
 * @license    http://www.gnu.org/licenses/lgpl-3.0.html  LGPL
 */
'use strict';

Node.prototype.getLayout = function () {
	if (this.typ === "threedots") {
		return top.treeData.node_layouts.threedots;
	}
	var layout_key = (this.typ === "group" && this.contenttype !== "text/weCollection" ? "group" : "item") +
		(this.selected ? "Selected" : "") +
		(this.disabled ? "Disabled" : "") +
		(this.checked ? "Checked" : "") +
		(this.open ? "Open" : "") +
		(this.typ === "item" && this.published === 0 ? "Notpublished" : "") +
		(this.typ === "item" && this.published === -1 ? "Changed" : "");

	return top.treeData.node_layouts[layout_key];
};

container.prototype.openClose = function (id) {
	if (id === "") {
		return;
	}
	var eintragsIndex = this.indexOfEntry(id);
	var openstatus = (this.elements[eintragsIndex].open ? 0 : 1);
	this.elements[eintragsIndex].open = openstatus;
	if (openstatus && !this.elements[eintragsIndex].loaded) {
		window.we_cmd("loadFolder", this.table, this.elements[eintragsIndex].id);
	} else {
		window.we_cmd("closeFolder", this.table, this.elements[eintragsIndex].id);
		drawTree();
	}
	if (openstatus) {
		this.elements[eintragsIndex].loaded = true;
	}
};

function info(text) {
	var t = document.getElementById("infoField");
	var s = document.getElementById("search");
	if (text !== " ") {
		s.style.display = "none";
		t.innerHTML = text;
		t.style.display = "block";
	} else {
		s.style.display = "block";
		t.style.display = "none";
		t.innerHTML = text;
	}
}

container.prototype.doClick = function (id) {
	var node = this.get(id);
	var ct = node.contenttype,
		table = node.table;

	id = node.we_id ? node.we_id : id;
	treeData.setScrollY();

	if(this.wasdblclick && node.typ === 'group'){
		this.openClose(id);
		return;
	}

	switch (table) {
		case WE().consts.tables.FILE_TABLE:
			if (this.wasdblclick && ct !== WE().consts.contentTypes.FOLDER_DOCUMENT) {
				WE().layout.openBrowser(id);
				window.setTimeout(function () {
					this.wasdblclick = false;
				}, 400);
				break;
			}
			/* falls through */
		default:
			WE().layout.weEditorFrameController.openDocument(table, id, ct);
			break;
	}
};


function initTree() {
	top.treeData = new container();
	top.treeData.table = WE().consts.tables.FILE_TABLE;
}
