/* global WE, errorHandler,treeData,drawTree, top */

/**
 * webEdition CMS
 *
 * webEdition CMS
 * $Author: Lukas Imhof $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

WE().layout = {
	editors: {
		CodeMirror: {}
	},
	ImageEditTools: null,
	TabView: null,
	browserwind: null,
	button: null,
	checkInitImageEditTools: null,
	cockpitFrame: null,
	collection: null,
	combobox: null,
	jsWindow: null,
	mainFrame: window,
	messageConsole: null,
	messageConsoleView: null,
	navigationHistory: null,
	overlay: null,
	isOverlay: null,
	getOverlayWindow: null,
	regular_logout: false,
	sidebar: null,
	toggleBox: null,
	weSuggest: null,
	windows: [],
	fileupload: {
		ExifReader: null,
		encodehunks: null,
		extractChunks: null,
		pica: null,
		getFileUploadGlobal: null,
	},
	dragNDrop: {
		dataTransfer: {
			text: ''
		}
	},
	openToEdit: function (tab, id, contentType) {
		if (id > 0) {
			WE().layout.weEditorFrameController.openDocument(tab, id, contentType);
		}
	},
	we_setPath: function (_EditorFrame, path, text, id, classname) {
		_EditorFrame = _EditorFrame ? _EditorFrame : WE().layout.weEditorFrameController.getActiveEditorFrame();
		// update document-tab
		_EditorFrame.initEditorFrameData({
			EditorDocumentText: text,
			EditorDocumentPath: path
		});

		if (classname) {
			WE().layout.multiTabs.setTextClass(_EditorFrame.FrameId, classname);
		}
		if (_EditorFrame.getDocumentReference().frames.editHeader === undefined) {
			return;
		}
		var doc = _EditorFrame.getDocumentReference().frames.editHeader.document;
		if (doc) {
			var div = doc.getElementById('h_path');
			if (div) {
				div.innerHTML = path.replace(/</g, '&lt;').replace(/>/g, '&gt;');
				if (id > 0) {
					div = doc.getElementById('h_id');
					div.innerHTML = id;
				}
			}
		}
	},
	reloadUsedEditors: function (reloadTabs) {
		var usedEditors = WE().layout.weEditorFrameController.getEditorsInUse();
		for (var frameId in usedEditors) {
			if (usedEditors[frameId].getEditorEditorTable() && reloadTabs[usedEditors[frameId].getEditorEditorTable()] &&
					((reloadTabs[usedEditors[frameId].getEditorEditorTable()]).indexOf(usedEditors[frameId].getEditorDocumentId()) !== -1 ||
					(reloadTabs[usedEditors[frameId].getEditorEditorTable()]).indexOf(String(usedEditors[frameId].getEditorDocumentId())) !== -1)) {
				if (usedEditors[frameId].getEditorIsActive()) {
					usedEditors[frameId].setEditorReloadAllNeeded(true);
					usedEditors[frameId].setEditorIsActive(true);
				} else {
					usedEditors[frameId].setEditorReloadAllNeeded(true);
				}
			}
		}
	},
	makeNewDoc: function (_EditorFrame, curType, curID, savetmpl, noClose) {
		var _EditorFrameDocumentRef = _EditorFrame.getDocumentReference();

		switch (curType) {
			case WE().consts.contentTypes.TEMPLATE:
				if (WE().util.hasPerm("NEW_WEBEDITIONSITE")) {
					if (_EditorFrame.getEditorMakeNewDoc()) {
						if (savetmpl) {
							top.we_cmd('new', WE().consts.tables.FILE_TABLE, '', WE().consts.contentTypes.WEDOCUMENT, '', curID);
						}
					} else if (noClose && _EditorFrame.getEditorIsInUse()) {
						_EditorFrameDocumentRef.frames.editHeader.location.reload();
					}
				}
				break;
			case WE().consts.contentTypes.OBJECT:
				if (WE().util.hasPerm('NEW_OBJECTFILE')) {
					if (_EditorFrame.getEditorMakeNewDoc()) {
						top.we_cmd('new', WE().consts.tables.OBJECT_FILES_TABLE, '', WE().consts.contentTypes.OBJECT_FILE, curID);
					} else if (noClose && _EditorFrame.getEditorIsInUse()) {
						_EditorFrameDocumentRef.frames.editHeader.location.reload();
					}

				}
		}
	},
	propEdit: {
		switchExt: function (win, id, name) {
			var a = win.document.we_form.elements;
			var ok = true;
			var prefix = name ? 'we_' + name + '_' : '';

			if (id) {
				ok = win.confirm(WE().consts.g_l.main.confirm_ext_change);
			}
			if (ok) {
				a[prefix + 'Extension'].value = (parseInt(a[prefix + 'IsDynamic'].value) ? WE().consts.global.DEFAULT_DYNAMIC_EXT : WE().consts.global.DEFAULT_STATIC_EXT);
			}
		}
	},
	checkFileUpload: function (we_cmd_args) {
		var parentObj = WE().layout.weEditorFrameController;
		var frame = WE().layout.weEditorFrameController.getVisibleEditorFrame();

		if (parentObj !== undefined && (frame.weFileUpload_instance) !== undefined && frame.weFileUpload_instance.getType() === 'binDoc') {
			frame.weFileUpload_instance.doUploadIfReady(function () {
				top.we_cmd.apply(window, we_cmd_args);
			});
		} else {
			top.we_cmd.apply(window, we_cmd_args);
		}

	},
	tree: {
		oldWidth: 0,
		widthBeforeDeleteMode: 0,
		getWidth: function () {
			var w = document.getElementById("bframeDiv").style.width;
			return w.substr(0, w.length - 2);
		},
		toggle: function (setVisible) {
			var tfd = window.document.getElementById("treeFrameDiv");
			var w = WE().layout.tree.getWidth();

			if (setVisible || (tfd.style.display === "none" && setVisible !== false)) {
				WE().layout.tree.oldWidth = (WE().layout.tree.oldWidth < WE().consts.size.tree.min ? WE().consts.size.tree.defaultWidth : WE().layout.tree.oldWidth);
				WE().layout.tree.setWidth(WE().layout.tree.oldWidth);
				tfd.style.display = "block";
				//setTreeArrow("left");
				WE().layout.tree.storeWidth(WE().layout.tree.oldWidth);
				return true;
			}
			tfd.style.display = "none";
			WE().layout.tree.oldWidth = w;
			WE().layout.tree.setWidth(WE().consts.size.tree.hidden);
			//setTreeArrow("right");
			return false;
		},
		treeInOut: function (out) {
			if (out && (WE().layout.tree.getWidth() <= WE().consts.size.tree.min)) {
				WE().layout.tree.toggle();
			} else if (!out && WE().layout.tree.getWidth() > WE().consts.size.tree.min) {
				WE().layout.tree.toggle();
			}
		},
		setWidth: function (w) {
			document.getElementById("bframeDiv").style.width = w + "px";
			document.getElementById("bm_content_frameDiv").style.left = w + "px";
			if (w > WE().consts.size.tree.hidden) {
				WE().layout.tree.storeWidth(w);
			}
		},
		inc: function () {
			var w = parseInt(WE().layout.tree.getWidth());
			if ((w >= WE().consts.size.tree.min) && (w < WE().consts.size.tree.max)) {
				w += WE().consts.size.tree.step;
				WE().layout.tree.setWidth(w);
			}
			if (w >= WE().consts.size.tree.max) {
				w = WE().consts.size.tree.max;
				WE().layout.tree.setWidth(w);
				window.document.getElementById("incBaum").style.backgroundColor = "grey";
			} else
			if (w < WE().consts.size.tree.min) {
				WE().layout.tree.toggle(true);
			}
		},
		dec: function () {
			var w = parseInt(WE().layout.tree.getWidth());
			w -= WE().consts.size.tree.step;
			if (w > WE().consts.size.tree.min) {
				WE().layout.tree.setWidth(w);
				document.getElementById("incBaum").style.backgroundColor = "";
			}
			if (w <= WE().consts.size.tree.min && ((w + WE().consts.size.tree.step) >= WE().consts.size.tree.min)) {
				WE().layout.tree.toggle();
			}
		},
		storeWidth: function (w) {
			var ablauf = new Date();
			var newTime = ablauf.getTime() + 30758400000;
			ablauf.setTime(newTime);
			WE().util.weSetCookie(document, "treewidth_main", w, ablauf, WE().consts.dirs.WEBEDITION_DIR);
		},
		setSearchButtonIcon: function (frame, table) {
			var button = frame.document.getElementById('searchButtonTree'),
				value = frame.document.getElementById('searchInputTree').value;

			if (value.match(/^#[0-9]+$/)) {
				button.children[0].className = "fa " + WE().consts.g_l.button.icon.openToEdit;
				button.style.cursor = 'pointer';
				return;
			}
			switch (table) {
				default:
					button.children[0].className = "fa " + WE().consts.g_l.button.icon.search;
					button.title = WE().consts.g_l.button.search;
					button.style.cursor = 'pointer';
					break;
				case WE().consts.tables.VFILE_TABLE:
					button.children[0].className = "fa fa-lg fa-close";
					button.title = '';
					button.style.cursor = 'default';
			}
		},
		handleSearch: function (value, table) {
			var found = value.match(/^#([0-9]+)$/);
			if (found) {
				WE().layout.weEditorFrameController.openDocument(table, found[1], '');
			} else {
				switch (table) {
					case WE().consts.tables.FILE_TABLE:
					case WE().consts.tables.TEMPLATES_TABLE:
					case WE().consts.tables.OBJECT_FILES_TABLE:
					case WE().consts.tables.OBJECT_TABLE:
						top.we_cmd('we_search_edit', value, table);
				}
			}
		},
		handleSearchMod: function (win, value) {
			var found = value.match(/^#([0-9]+)$/);
			if (found) {
				win.document.defaultView.doClick(found[1]);
			} else {
				win.we_cmd('show_search', value);
			}
		}
	},
	vtab: {
		click: function (tab, table) {
			if (WE().session.deleteMode) {
				we_cmd('exit_delete', table);
			}
			if (tab.classList.contains("tabActive")) {
				if (WE().layout.tree.toggle()) {
					we_cmd('loadVTab', table, 0);
				}
			} else {
				WE().layout.vtab.setActiveTab(table);
				WE().layout.tree.treeInOut(true);
				we_cmd('loadVTab', table, 0);
			}
		},
		setActiveTab: function (table) {
			var allTabs = document.getElementById("vtabs").getElementsByClassName("tab");
			for (var i = 0; i < allTabs.length; i++) {
				allTabs[i].className = "tab " + (allTabs[i].getAttribute("data-table") === table ? "tabActive" : "tabNorm");
			}
		}
	},
	openBrowser: function (url) {
		if (!url) {
			url = "/";
		}
		try {
			WE().layout.browserwind = window.open(WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=openBrowser&" + (isNaN(url) ? "url=" + encodeURI(url) : "id=" + url), "browser", "menubar=yes,resizable=yes,scrollbars=yes,location=yes,status=yes,toolbar=yes");
		} catch (e) {
			WE().util.showMessage(WE().consts.g_l.alert.browser_crashed, WE().consts.message.WE_MESSAGE_ERROR, window);
		}
	},
	pushCmdToModule: function (args) {
		var wind = WE().layout.jsWindow.prototype.find('edit_module');
		WE().t_e('command pushed to module ', args[0]);
		if (wind) {
			wind.content.we_cmd(args[0]);
			if (args[0] !== "empty_log") {
				wind.focus();
			}
		}
	},
	maskFrame: function (target) {
		var parent, mask;
		switch (target) {
			case 'multieditor':
				parent = top.document.getElementById('bm_content_frameDiv');
				break;
			case 'sidebar':
				parent = top.document.getElementById('sidebarDiv');
				break;
			default:
				return;
		}
		if (parent && !top.document.getElementById('frameMask_' + target)) {
			mask = document.createElement('DIV');
			mask.className = 'frameMask';
			mask.id = 'frameMask_' + target;
			parent.appendChild(mask);
		}
	},
	unmaskFrame: function (target) {
		var mask;
		switch (target) {
			case 'multieditor':
			case 'sidebar':
				mask = top.document.getElementById('frameMask_' + target);
				break;
			default:
				return;
		}
		if (mask) {
			mask.parentNode.removeChild(mask);
		}
	}
};