/* global WE, top */

/**
 * webEdition CMS
 *
 * webEdition CMS
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';
var progressBar = WE().util.getDynamicVar(document, 'loadVarProgressBar', 'data-progress');

function setProgressText(name, text) {
	var div = document.getElementById(name);
	if (div) {
		div.innerHTML = text;
	}
}

function setProgress(progress, name) {
	name = name ? name : '';
	var elems = document.getElementsByClassName('progressbar');
	for (var i = 0; i < elems.length; i++) {
		elems[i].style.display = "";
	}
	var koef = progressBar[name].koef;
	document.getElementById("progress_image" + name).style.width = (koef * progress) + "px";
	document.getElementById("progress_image_bg" + name).style.width = (koef * 100) - (koef * progress) + "px";
	setProgressText("progress_text" + name, (Math.round(progress * 10) / 10) + "%");
	updateEst(name, progress);
}

function hideProgress() {
	var elems = document.getElementsByClassName('progressbar');
	for (var i = 0; i < elems.length; i++) {
		elems[i].style.display = "none";
	}
}

function updateEst(name, progress) {
	if (parseFloat(progress) === 100) {
		progressBar[name].count = 1;
		progressBar[name].est = 0;
		return;
	}
	if (parseFloat(progress) === 0) {
		progressBar[name].start = Date.now();
		progressBar[name].lastTime = Date.now();
		progressBar[name].lastProgress = 0;
	}

	var diff = {
		time: Date.now() - progressBar[name].start,
		timeLast: Date.now() - progressBar[name].lastTime,
		progressTodo: 100 - progress,
		progressDiff: progress - progressBar[name].lastProgress
	};
	var estFull = (progress > 0 ? (((diff.time * 100) / progress) - diff.time) : 0);
	var estLast = (diff.progressDiff > 0 ? (diff.timeLast * diff.progressTodo / diff.progressDiff) : estFull);

	progressBar[name].est = (progress > 0 ? (estFull + estLast) / 2 : -1);
	progressBar[name].count = 30;
	if (!progressBar[name].timer) {
		updateElapsed(name);
		progressBar[name].timer = window.setInterval(updateElapsed, 1000, name);
	}
	progressBar[name].lastTime = Date.now();
	progressBar[name].lastProgress = progress;
}

function updateElapsed(name) {
	progressBar[name].count--;

	if (progressBar[name].count < 0) {
		window.clearInterval(progressBar[name].timer);
		progressBar[name].timer = 0;
	}
	var elapsed = new Date();
	elapsed.setTime(-3600000 + Date.now() - progressBar[name].start);
	progressBar[name].est -= 1000;
	var est = new Date();
	est.setTime(-3600000 + (progressBar[name].est > 0 ? progressBar[name].est : 0));
	setProgressText("elapsedTime" + name, elapsed.toLocaleTimeString() + " / " + (progressBar[name].est >= 0 ? est.toLocaleTimeString() : '<i class="fa fa-cog fa-spin"></i>')
		);
}
