/* global tinymce */

/**
 * webEdition CMS
 *
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

(function () {
	var wevisualaid = (function () {
		var global = tinymce.util.Tools.resolve('tinymce.PluginManager');
		var weToggleBorders = function (ed) {
			var settings = ed.settings,
				dom = ed.dom;

			settings.weHasVisual = !settings.weHasVisual;

			ed.execCommand('mceVisualBlocks');
			ed.execCommand('mceVisualChars');
			ed.execCommand('mceToggleVisualAid');

			ed.weControlManager.setActive('wevisualaid', settings.weHasVisual);

			var nodes = dom.select('acronym,span,abbr,table');
			if (nodes.length > 0) {
				tinymce.each(nodes, function (node) {
					switch (node.nodeName) {
						case 'TABLE':
							var attrib = dom.getAttrib(node, 'border');
							if (!attrib || parseInt(attrib) === 0) {
								dom.addClass(node, 'mce-item-table');
							} else {
								dom.removeClass(node, 'mce-item-table');
							}
							return;
						case 'ACRONYM':
							if (settings.weHasVisual) {
								dom.addClass(node, 'mceItemWeAcronym');
							} else {
								dom.removeClass(node, 'mceItemWeAcronym');
							}
							return;
						case 'ABBR':
							if (settings.weHasVisual) {
								dom.addClass(node, 'mceItemWeAbbr');
							} else {
								dom.removeClass(node, 'mceItemWeAbbr');
							}
							return;
						case 'SPAN':
							var attrib = dom.getAttrib(node, 'lang');
							if (attrib) {
								if (settings.weHasVisual) {
									dom.addClass(node, 'mceItemWeLang');
								} else {
									dom.removeClass(node, 'mceItemWeLang');
								}
							}
							return;
					}
				});

			}
		};

		var Commands = {register: function (editor) {
				editor.addCommand('mceWevisualaid', function () {
					weToggleBorders(editor);
				});
			}};

		var Buttons = {register: function (editor) {
				editor.ui.registry.addToggleButton('wevisualaid', {
					icon: 'paragraph',
					tooltip: WE().consts.g_l.tinyMceTranslationObject.tt_wevisualaid,
					onAction: function () {
						editor.execCommand('mceWevisualaid');
					},
					onSetup: function (buttonApi) {
						editor.weControlManager.register('button', 'wevisualaid', buttonApi);
						editor.settings.weHasVisual = false;
					}
				});
			}};

		global.add('wevisualaid', function (editor) {
			Commands.register(editor);
			Buttons.register(editor);
			return {
				initTableBorders: function (ed) {
					var dom = ed.dom,
						nodes = dom.select('table');

					if (nodes.length > 0) {
						tinymce.each(nodes, function (node) {
							var attrib = dom.getAttrib(node, 'border');
							if (!attrib || parseInt(attrib) === 0) {
								dom.addClass(node, 'mce-item-table');
							} else {
								dom.removeClass(node, 'mce-item-table');
							}
						});
					}
				}
			};
		});

		function Plugin() {}

		return Plugin;

	}());
})();